import logging

from pylons import g
from mpulsweb.lib.translation import _
from pylons import c
from pylons.controllers.util import abort

from mpulsweb.lib.security import checkRole
from mpulsweb.lib.base import BaseController, render

log = logging.getLogger(__name__)

DEFAULT_REQUIRED_TEXT = _('This field is a required field, and must be filled out')
DEFAULT_EVALUATION_TEXT = _('This field is relevant for evaluation purpose')

class AnnotationsController(BaseController):

    @checkRole(('admin_ka', 'cm_ka', 'pt_dlr'))
    def help(self, id):
        """Show help for the given id.
        The id is first looked up in the formed tree, because its node
        components also have a description that can be used as a title.
        However, not all help texts refer to formed tree components (for
        instances 'wiederaufnahme' and 'neuaufnahme'), so helpData is
        searched directly as well.
        """
        nc = g.formedTree.findByName(id)
        if nc is not None:
            text = nc.getHelp()
            title = nc.getDescription()
        else:
            text = g.helpData.getHelp(id)
            if text is None:
                abort(404, _('No help available'))
            title = ""
        c.annotation_header = _('Help')
        c.annotation_title = title
        c.annotation_text = text
        return render('/annotations/annotation.mako')

    @checkRole(('admin_ka', 'cm_ka', 'pt_dlr'))
    def required(self, id):
        nc = g.formedTree.findByName(id)
        if not nc:
            abort(404, _('No help available'))
        c.annotation_header = _('Required field')
        c.annotation_title = nc.getDescription()
        c.annotation_text = nc.getRequired() or DEFAULT_REQUIRED_TEXT 
        return render('/annotations/annotation.mako')

    @checkRole(('admin_ka', 'cm_ka', 'pt_dlr'))
    def evaluation(self, id):
        nc = g.formedTree.findByName(id)
        if not nc:
            abort(404, _('No help available'))
        c.annotation_header = _('Evaluation relevant field')
        c.annotation_title = nc.getDescription()
        c.annotation_text = nc.getEvaluation() or \
            DEFAULT_EVALUATION_TEXT
        return render('/annotations/annotation.mako')
