# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS -
# Web-Anwendungs-Server fuer Kompetenzagenturen).
#
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls WASKA has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.

"""The application's Globals object"""

import os
import logging

from paste.reloader import watch_file
from paste.registry import StackedObjectProxy

from pylons import config

from formed.meta.io.document import openDocument

from libmpuls.service.tags import TagSetterConfig

from mpulsweb.config.importer import import_overridable_module
from mpulsweb.lib.timelog import setupTimeLogging
from mpulsweb.lib.config import MpulsAppConfig, MpulsDBMapping, MpulsDBList
from mpulsweb.model.annotations import AnnotationsProvider


log = logging.getLogger(__name__)


# MPuls session object.  Application specific wrapper around the pylons
# session object.  The actual functionality is defined in
# mpulsweb/lib/session.py.  It's set on a per-request basis in
# mpulsweb/lib/base.py.  It's located here in app_globals.py to avoid
# excessive problems with circular imports.
session = StackedObjectProxy()


class Globals(object):

    """Globals acts as a container for objects available throughout the
    life of the application
    """

    def __init__(self):
        """One instance of Globals is created during application
        initialization and is available during requests via the 'g'
        variable
        """
        def filename_from_config(setting, default):
            return os.path.join(config.get('pylons.paths').get('root'),
                                '..', config.get(setting, default))

        # Load mpuls configuration
        config_file = filename_from_config('mpuls.app.config',
                                           'default_config.json')
        mapping_file = filename_from_config('mpuls.app.dbmapping',
                                            'default_dbmapping.json')
        tagging_file = filename_from_config('mpuls.app.tagging',
                                            'default_tagging.xml')
        dblist_file = filename_from_config('mpuls.app.dblist',
                                            'default_dblist.json')

        self.config_path = os.path.dirname(config_file)
        self.mpuls_config = MpulsAppConfig(config_file)
        self.mpuls_dbmapping = MpulsDBMapping(mapping_file)
        self.mpuls_dblist = MpulsDBList(dblist_file)

        # Load formed from defined location. If no location is defined. Take
        # the default which is the formed dir under public.
        formed_dir = config.get('mpuls.app.path.formed',
            os.path.join(config.get('pylons.paths').get('static_files')[0],
                         'formed'))
        formed_file = os.path.join(formed_dir,
                                   self.mpuls_config.get('formed', 'treefile'))
        annotation_file = os.path.join(formed_dir,
                                       self.mpuls_config.get('formed',
                                                             'annotation'))

        # Build FormedTree. This tree will work as template for all cases
        log.debug("loading formed definition from %r", formed_file)
        self.formedTree = openDocument(formed_file)
        self.helpData = AnnotationsProvider(annotation_file)

        # Load taggingconfig
        log.debug("Loading configuration for tagging from %s" % tagging_file)
        self.taggingConfFile = tagging_file
        self.taggingConf = TagSetterConfig(tagging_file) 

        # tell paste to restart the server if the formed files change
        watch_file(formed_file)
        watch_file(annotation_file)
        watch_file(tagging_file)

        # TODO: These setups should be while application intialisation, as they
        # are not related to any global vars or functions. Move to
        # config/environment.py

        # set time logging
        time_host = config.get('time_host')
        time_port = config.get('time_port')

        if time_host and time_port:
            setupTimeLogging(time_host, int(time_port))
        elif time_host:
            setupTimeLogging(int(time_host))

        self._init_case()

    def _init_case(self):
        """Set the mpuls application specific case classes"""
        case_module = import_overridable_module("model.case")
        self.case = case_module.Case
        self.case.set_fields(self.mpuls_config.get("case", "fields"))
        self.case_factory = case_module.CaseFactory(self.case)
        search_module = import_overridable_module("lib.search")
        self.case_search = search_module.CaseSearch(
            self.case_factory.get_search_retrieve_fields(),
            self.case_factory.get_search_match_fields())
