INSERT INTO art_beendigung_tbl VALUES (-3, 'unbekannt'); 

-- 
-- If CM Case
-- - If datum_feststellung is unset: set to erstgespraech
-- - If beendigung_5 is unset:
--      - if s.zugriff > '2008-09-22 19:35': set to s.zugriff 
--        else: set to '2008-08-31'
-- - If cm_end_art is unset (i.e. = -1): set to "unknown" (-3)
-- - If phase < 3: Set to 3
-- - If phase > 3: 
--      - if beendigung_nachbetreuung is unset:                
--                              set beendigung_nachbetreuung to s.zugriff
--      - if phase = 4: Set phase to 5
--
-- If Beratung
-- - if datum_feststellung is unset: set to erstgespraech
-- - if datum_ende_beratung is unset:
--      - if s.zugriff > '2008-09-22 19:35': set to s.zugriff 
--        else: set to '2008-08-31'
-- - if phase = 6 or phase < 2: Set to 7
--



-- Set cm_end_art = "unknown" and datum_cm_ende = parameter rec_date 
CREATE OR REPLACE FUNCTION set_cm_ende(rec_id int, rec_date timestamp) 
RETURNS void AS
$$
BEGIN
    UPDATE master_tbl
        SET beendigung_5 = rec_date
        WHERE id = rec_id 
            AND datum_feststellung <= rec_date;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION set_cm_ende_art(rec_id int, rec_art int) 
RETURNS void AS
$$
BEGIN
    UPDATE master_tbl 
        SET art_beendigung = rec_art
        WHERE id = rec_id;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION set_datum_feststellung(rec_id int, rec_date timestamp) 
RETURNS void AS
$$
BEGIN
    UPDATE master_tbl
        SET datum_feststellung = rec_date
        WHERE id = rec_id 
            AND datum_feststellung is NULL;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION set_phase(rec_id int, rec_phase int) 
RETURNS void AS
$$
BEGIN
    UPDATE master_tbl
        SET phase = rec_phase
        WHERE id = rec_id ;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION set_nbtr_ende(rec_id int, rec_date timestamp) 
RETURNS void AS
$$
BEGIN
    UPDATE master_tbl 
        SET beendigung_nachbetreuung = rec_date
        WHERE id = rec_id
            AND beendigung_5 <= rec_date;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION set_beratung_ende(rec_id int, rec_date timestamp) 
RETURNS void AS
$$
BEGIN
    UPDATE master_tbl 
        SET datum_ende_beratung = rec_date
        WHERE id = rec_id
            AND datum_feststellung < rec_date;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION delete_case(rec_id int) RETURNS void AS
$$
BEGIN
    DELETE FROM rg_kompetenzfestellung_tbl where master_id = rec_id;
    DELETE FROM rg_angebote_berufliche_qualifizierung_tbl where master_id = rec_id; 
    DELETE FROM rg_angebote_bildenden_bereich_tbl where master_id = rec_id;
    DELETE FROM rg_angebote_lebensbewaeltigung_tbl where master_id = rec_id;
    DELETE FROM rg_angebote_berufsvorbereitung_tbl where master_id = rec_id;
    DELETE FROM master_tbl where id = rec_id;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION is_cm(rec_id int) RETURNS BOOLEAN AS
$$
DECLARE
    result boolean;
BEGIN
    SELECT INTO result 
            (kompetenzfeststellung = '1' 
            OR (kompetenzfeststellung = '-1' and phase in (2,3,4,5)) 
            OR (bereich_berufliche_qualifizierung = '1' 
                    or bereich_berufsvorbereitung = '1'
                    or bildender_bereich = '1' 
                    or bereich_lebensbewaeltigung = '1' 
                    or sonstiger_foerderbedarf = '1'))
            FROM master_tbl WHERE id = rec_id;
    RETURN result;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION is_beratung(rec_id int) RETURNS BOOLEAN AS
$$
DECLARE
    result boolean;
BEGIN
    SELECT INTO result 
            (kompetenzfeststellung = '0' 
            OR (kompetenzfeststellung = '-1' and phase in (6,7))
	        OR (begleitung in (-2,0,1,2,3)) and phase in (-1,0,1,6,7))
            FROM master_tbl WHERE id = rec_id;
    RETURN result;
END;
$$ LANGUAGE plpgsql;


ALTER TABLE master_tbl DISABLE TRIGGER ALL;

-- Loesche alle Anonymisierten Fallakten ohne erstgespraech
SELECT delete_case(m.id) 
    FROM master_tbl m, ka_status_tbl s 
    WHERE m.id = s.master_id
        AND m.erstgespraech is NULL 
        AND s.status = 5;

-- CM Case
-- - If datum_feststellung is unset: set to erstgespraech
SELECT set_datum_feststellung(m.id, m.erstgespraech)
    FROM master_tbl m, ka_status_tbl s 
    WHERE m.id = s.master_id
        AND is_cm(m.id)
        AND datum_feststellung is NULL
        AND s.status = 5;

-- - If beendigung_5 is unset:
--      - if s.zugriff > '2008-09-22 19:35': set to s.zugriff 
SELECT set_cm_ende(m.id, s.zugriff)
    FROM master_tbl m, ka_status_tbl s 
    WHERE m.id = s.master_id
        AND is_cm(m.id) 
        AND m.beendigung_5 is NULL
        AND s.zugriff >=  '2008-09-22 19:35:00'::timestamp 
        AND s.status = 5;

--        else: set to '2008-08-31'
SELECT set_cm_ende(m.id, '2008-08-31'::timestamp)
    FROM master_tbl m, ka_status_tbl s 
    WHERE m.id = s.master_id
        AND is_cm(m.id) 
        AND m.beendigung_5 is NULL
        AND s.zugriff <  '2008-09-22 19:35:00'::timestamp 
        AND s.status = 5;

-- - If cm_end_art is unset (i.e. = -1): set to "unknown" (-3)
SELECT set_cm_ende_art(m.id, -3)
    FROM master_tbl m, ka_status_tbl s 
    WHERE m.id = s.master_id
        AND is_cm(m.id)
        AND art_beendigung = -1
        AND s.status = 5;

-- - If phase < 3: Set to 3
SELECT set_phase(m.id, 3)
    FROM master_tbl m, ka_status_tbl s 
    WHERE m.id = s.master_id
        AND is_cm(m.id)
        AND phase < 3
        AND s.status = 5;

-- - If phase > 3: 
--      - if beendigung_nachbetreuung is unset:                
--                              set beendigung_nachbetreuung to s.zugriff
SELECT set_nbtr_ende(m.id, s.zugriff)
    FROM master_tbl m, ka_status_tbl s 
    WHERE m.id = s.master_id
        AND is_cm(m.id)
        AND phase > 3
        AND beendigung_nachbetreuung is NULL
        AND s.zugriff >=  '2008-09-22 19:35:00'::timestamp 
        AND s.status = 5;

SELECT set_nbtr_ende(m.id, '2008-08-31'::timestamp)
    FROM master_tbl m, ka_status_tbl s 
    WHERE m.id = s.master_id
        AND is_cm(m.id)
        AND phase > 3
        AND beendigung_nachbetreuung is NULL
        AND s.zugriff <  '2008-09-22 19:35:00'::timestamp 
        AND s.status = 5;

--      - if phase = 4: Set phase to 5
SELECT set_phase(m.id, 5)
    FROM master_tbl m, ka_status_tbl s 
    WHERE m.id = s.master_id
        AND is_cm(m.id)
        AND phase =4
        AND s.status = 5;

--
-- If Beratung
-- - if datum_feststellung is unset: set to erstgespraech
SELECT set_datum_feststellung(m.id, m.erstgespraech)
    FROM master_tbl m, ka_status_tbl s 
    WHERE m.id = s.master_id
        AND is_beratung(m.id)
        AND datum_feststellung is NULL
        AND s.status = 5;

-- - if datum_ende_beratung is unset:
--      - if s.zugriff > '2008-09-22 19:35': set to s.zugriff
SELECT set_beratung_ende(m.id, s.zugriff)
    FROM master_tbl m, ka_status_tbl s 
    WHERE m.id = s.master_id
        AND is_beratung(m.id) 
        AND m.datum_ende_beratung is NULL
        AND s.zugriff >=  '2008-09-22 19:35:00'::timestamp 
        AND s.status = 5;

--        else: set to '2008-08-31'
SELECT set_beratung_ende(m.id, '2008-08-31'::timestamp)
    FROM master_tbl m, ka_status_tbl s 
    WHERE m.id = s.master_id
        AND is_beratung(m.id) 
        AND m.datum_ende_beratung is NULL
        AND s.zugriff <  '2008-09-22 19:35:00'::timestamp 
        AND s.status = 5;

-- - if phase = 6 or phase < 2: Set to 7
SELECT set_phase(m.id, 7)
    FROM master_tbl m, ka_status_tbl s 
    WHERE m.id = s.master_id
        AND is_beratung(m.id)
        AND (phase = 6 or phase < 2)
        AND s.status = 5;

-- Setze alle intersexuelle auf keine Angabe (Wunsch von Herrn Gregersen):
UPDATE master_tbl SET geschlecht = -1 WHERE geschlecht = 2;
DELETE FROM geschlecht_tbl WHERE id = 2;

-- Clean Up
DROP FUNCTION set_cm_ende(rec_id int, rec_date timestamp);
DROP FUNCTION set_cm_ende_art(rec_id int, rec_art int) ;
DROP FUNCTION set_datum_feststellung(rec_id int, rec_date timestamp) ;
DROP FUNCTION set_phase(rec_id int, rec_phase int) ;
DROP FUNCTION set_nbtr_ende(rec_id int, rec_date timestamp) ;
DROP FUNCTION set_beratung_ende(rec_id int, rec_date timestamp) ;
DROP FUNCTION delete_case(rec_id int);
DROP FUNCTION is_cm(rec_id int);
DROP FUNCTION is_beratung(rec_id int);

ALTER TABLE master_tbl ENABLE TRIGGER ALL;

