-- Hier finden sich zusätzliche Tabellen, die nicht durch den Formed generiert
-- werden.
--
-- $Revision: 4b4712edc9ce $

CREATE TABLE ka_konfiguration_tbl
(
	id serial NOT NULL,
	bez varchar(64) NOT NULL,
	wert text,
	CONSTRAINT ka_konfiguration_tbl_pkey PRIMARY KEY (id)
);

INSERT INTO ka_konfiguration_tbl (bez, wert) VALUES ('fkz', '');
INSERT INTO ka_konfiguration_tbl (bez, wert) VALUES ('ka_name', '');
INSERT INTO ka_konfiguration_tbl (bez, wert) VALUES ('ka_adresse', '');
INSERT INTO ka_konfiguration_tbl (bez, wert) VALUES ('ee_formular', '');
INSERT INTO ka_konfiguration_tbl (bez, wert) VALUES ('fp_formular', '');
INSERT INTO ka_konfiguration_tbl (bez, wert) VALUES ('ud_formular', '');
INSERT INTO ka_konfiguration_tbl (bez, wert) VALUES ('anon_transfer', 'on');
INSERT INTO ka_konfiguration_tbl (bez, wert) VALUES ('max_speicherdauer', '365');
INSERT INTO ka_konfiguration_tbl (bez, wert) VALUES ('jmdstrukur_permission_userid', '');

CREATE TABLE ka_dokumente_tbl
(
	id serial NOT NULL,
    	uuid_id CHAR(36) UNIQUE,
	master_id int4 REFERENCES master_tbl(id) ON DELETE CASCADE,
	groesse int4,
	mime varchar(64),
	daten bytea, 
	name varchar(128),
	CONSTRAINT ka_dokumente_tbl_pkey PRIMARY KEY (id)
);

-- ka_status_tbl
--
-- Speichert Zustand und letzte Zugriffszeit der Fallakte.
-- Mögliche Zustände:
--	Offen (1) 
--	Geschlossen (2)
--	Schwebend-Gelöscht (3)
--	Schwebend-Anonymisiert (4)
--	Anonymsiert (5)

CREATE TABLE ka_status_tbl
(
	id serial NOT NULL,
	master_id int4 REFERENCES master_tbl(id) ON DELETE CASCADE,
	status    int4,
	zugriff   timestamp,
	erstellt  timestamp DEFAULT now(),
	anonymisiert timestamp,
	CONSTRAINT ka_status_tbl_pkey PRIMARY KEY (id)
);

-- ka_termine_tbl
-- 
-- Speichert grundlegende Angaben zu einem Termin. Weitergehende Angaben
-- sollten in einer extra Tabelle gespeichert werden.

CREATE TABLE ka_termine_tbl
(
	id serial NOT NULL,
	master_id int4 REFERENCES master_tbl(id) ON DELETE CASCADE,
	sdatum timestamp,
	edatum timestamp,
	name varchar(128),
	beschreibung text,
	art int4 DEFAULT 0, 
	CONSTRAINT ka_termine_tbl_pkey PRIMARY KEY (id)
);

-- ka_logbuch_art_tbl
--
-- Speichert die verschiedenen Arten von Logbuch-Eintraegen
--

CREATE TABLE ka_logbuch_art_tbl (
    id         SERIAL NOT NULL,
    beschreibung VARCHAR(256),
    CONSTRAINT ka_logbuch_art_tbl_pkey PRIMARY KEY (id)
);

GRANT SELECT ON ka_logbuch_art_tbl TO GROUP :cm_group;
GRANT SELECT ON ka_logbuch_art_tbl TO GROUP :admin_group;

-- Übernahme der Werte aus WASKA 1.7.1.
-- -2 | Sonstige Angaben
-- -1 | Keine Angabe
--  0 | Automatischer Eintrag 
--  1 | Gespräch mit dem Jugendlichen
--  2 | Korrespondenz mit dem Jugendlichen (Telefonate/Schriftverkehr)
--  3 | Begleitung des Jugendlichen zu externen Stellen
--  4 | Sonstige Betreuung des Jugendlichen
--  5 | Schriftverkehr
--  6 | Gespräch/Telefonat mit Kooperationspartnern
--  7 | Gespräch/Telefonat mit Dritten
--  8 | Förder- und Fallkonferenz 
-- 10 | Kompetenzfeststellung 
-- 11 | Gespräch mit Eltern/ Personensorgeberechtigten
-- 12 | Korrespondenz mit Eltern/ Personensorgeberechtigten (Telefonate/Schriftverkehr)
-- 13 | Sonstige Maßnahmen der Elternarbeit
-- 14 | Fahrtwege
-- 15 | Dokumentation WASKA
-- 16 | Teambesprechung

INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES  (-2, 'Sonstige Angaben');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES  (-1, 'Keine Angabe');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES  (0, 'Automatischer Eintrag');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES  (1, 'Gespräch mit dem Jugendlichen');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES  (2, 'Korrespondenz mit dem Jugendlichen (Telefonate/Schriftverkehr)');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES  (3, 'Begleitung des Jugendlichen zu externen Stellen');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES  (4, 'Sonstige Betreuung des Jugendlichen');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES  (5, 'Schriftverkehr');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES  (6, 'Gespräch/Telefonat mit Kooperationspartnern');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES  (7, 'Gespräch/Telefonat mit Dritten');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES  (8, 'Förder- und Fallkonferenz');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES  (10, 'Kompetenzfeststellung');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES  (11, 'Gespräch mit Eltern/ Personensorgeberechtigten');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES  (12, 'Korrespondenz mit Eltern/ Personensorgeberechtigten (Telefonate/Schriftverkehr)');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES  (13, 'Sonstige Maßnahmen der Elternarbeit');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES  (14, 'Fahrtwege');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES  (15, 'Dokumentation WASKA');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES  (16, 'Teambesprechung');

CREATE TABLE ka_logbuch_typ_tbl (
    id         SERIAL NOT NULL,
    beschreibung VARCHAR(256),
    CONSTRAINT ka_logbuch_typ_tbl_pkey PRIMARY KEY (id)
);

GRANT SELECT ON ka_logbuch_typ_tbl TO GROUP :cm_group;
GRANT SELECT ON ka_logbuch_typ_tbl TO GROUP :admin_group;

INSERT INTO ka_logbuch_typ_tbl (id, beschreibung) VALUES (0, 'nicht aufsuchend');
INSERT INTO ka_logbuch_typ_tbl (id, beschreibung) VALUES (1, 'aufsuchend');
INSERT INTO ka_logbuch_typ_tbl (id, beschreibung) VALUES (-1, 'Keine Angabe');

-- ka_logbuch_tbl
--
-- Verwaltet Logbucheintraege zu Fallakten
--

CREATE TABLE ka_logbuch_tbl (
    id         SERIAL NOT NULL,
    master_id  int4 REFERENCES master_tbl(id)         ON DELETE CASCADE,
    uuid_id    CHAR(36),
    datum      TIMESTAMP,
    bearbeiter VARCHAR(256),
    art        int4 REFERENCES ka_logbuch_art_tbl(id) ON DELETE SET DEFAULT,
    typ        int4 REFERENCES ka_logbuch_typ_tbl(id) ON DELETE SET DEFAULT,
    kurz_notiz VARCHAR(256),
    notiz      text,
    dauer      interval,
    UNIQUE (uuid_id),
    CONSTRAINT ka_logbuch_tbl_pkey PRIMARY KEY (id)
);

-- ka_nachrichten_tbl
--
-- Verwaltet Logbucheintraege zu Fallakten
--

CREATE TABLE ka_nachrichten_art_tbl (
    id         SERIAL NOT NULL,
    beschreibung VARCHAR(128),
    CONSTRAINT ka_nachrichten_art_tbl_pkey PRIMARY KEY (id)
);

INSERT INTO ka_nachrichten_art_tbl (id, beschreibung) VALUES ( -2, 'Sonstiges');
INSERT INTO ka_nachrichten_art_tbl (id, beschreibung) VALUES ( -1, 'Keine Angabe');
INSERT INTO ka_nachrichten_art_tbl (id, beschreibung) VALUES (  0, 'Systemnachricht');

CREATE TABLE ka_nachrichten_tbl (
    id         SERIAL NOT NULL,
    datum      TIMESTAMP,
    art        int4 REFERENCES ka_nachrichten_art_tbl(id) ON DELETE SET DEFAULT,
    titel      VARCHAR(128),
    nachricht  text,
    CONSTRAINT ka_nachrichten_tbl_pkey PRIMARY KEY (id)
);

CREATE TABLE nm_benutzer_nachrichten_tbl 
(
	id serial NOT NULL,
	benutzer_id int4 REFERENCES ka_benutzer_tbl(id) ON DELETE CASCADE,
	nachrichten_id int4 REFERENCES ka_nachrichten_tbl(id) ON DELETE CASCADE,
	gelesen timestamp DEFAULT now(),
	CONSTRAINT nm_benutzer_news_pkey PRIMARY KEY (id)
);

-- ka_tags_tbl
-- Enthaelt tags, die an Fallakten gehangen werden koennen

CREATE TABLE tags (
    id          SERIAL PRIMARY KEY,
    label       VARCHAR(256) NOT NULL,
    description TEXT,
    systemtag   BOOLEAN DEFAULT TRUE
);

CREATE TABLE nm_tags_cases ( 
    id SERIAL PRIMARY KEY,
    master_id   integer NOT NULL REFERENCES master_tbl(id) ON DELETE CASCADE,
    tag_id    integer NOT NULL REFERENCES tags(id)       ON DELETE CASCADE,
    created   TIMESTAMP DEFAULT now(),
    UNIQUE (master_id, tag_id)
);

-- Add systemtags

-- Autogenerated tag insert statements
DELETE FROM tags;
INSERT INTO tags (id, label, description) VALUES (0, 'Alle', 'Jede Fallakte hat einen Tag');
INSERT INTO tags (id, label, description) VALUES (1, 'Grundsätzlich auswertbar', 'Eine Fallakte ist grundsätzlich auswertbar und steht für Auswertungen zur Verfügung.');
INSERT INTO tags (id, label, description) VALUES (2, 'Berichtsfähig', 'Eine Fallakte ist grundsätzlich auswertbar. Sie befindet sich darüber hinaus mindestens im laufendem CM und enthält eine Geschlechtsangabe. Die Fallakte steht für Auswertungen wie den Verwendungsnachweis zur Verfügung');
INSERT INTO tags (id, label, description) VALUES (10, 'Datumsangaben konsistent', 'Alle Datumsangaben in der Fallakte befinden sich in chronologisch korrekter Reihenfolge und liegen nicht in der Zukunft');
INSERT INTO tags (id, label, description) VALUES (11, 'Phase ist gesetzt', 'Die Phase der Fallakte ist nicht unbekannt');
INSERT INTO tags (id, label, description) VALUES (12, 'Synchronität der Phasen ist gegeben', 'Die in der Fallakte angegebenen Datumsangaben sind nicht weiter ausgefüllt, als es die aktuell gesetzte Phase. Z.b ein gesetzten Beendigungsdatum bei laufender Phase');
INSERT INTO tags (id, label, description) VALUES (13, 'Junger Mensch ist ein CM-Kunde', 'Für den jungen Menschen wurde im Phasenmodell mindestens ein laufendes Case Management dokumentiert');
INSERT INTO tags (id, label, description) VALUES (14, 'Geschlechtsangabe angegeben', 'Die Geschlechtsangabe des jungen Menschen ist angegeben und hat nicht den Wert "unbekannt"');
INSERT INTO tags (id, label, description) VALUES (15, 'Formale Angaben "Übergangsprognose" in der Fallakte sind passend zur gesetzten Phase', 'Die Regeln, die sicherstellen das die Phase des CM nur bei bestimmten Angaben in der Übergangsprognose gestartet werden kann, sind erfüllt.');
INSERT INTO tags (id, label, description) VALUES (16, 'Formale Angabe "Übernahme CM" in der Fallakte sind passend zur gesetzten Phase', 'Die Regeln, die sicherstellen das die Phase CM bzw. Beratung nur dann gestartet werden können wenn eine entsprechende Notwendigkeit zur Übernahme in das CM dokumentiert wurde, sind erfüllt.');
INSERT INTO tags (id, label, description) VALUES (17, 'Die Dauer der Phasen überschreitet nicht die maximale Länge', 'Die Regeln, die sicherstellen das die Phasen nicht die maximale Länge überschreiten, sind erfüllt.');
INSERT INTO tags (id, label, description) VALUES (101, 'CM nur startbar wenn letzte Jahr und Prognose', 'Schüler/innen dürfen nur dann ins Case Managagement aufgenommen werden, wenn sie sich im letzten Schulbesuchsjahr befinden und eine Übergangsprognose vorliegt.');
INSERT INTO tags (id, label, description) VALUES (102, 'CM nur startbar wenn letzte Jahr und Prognose', 'Schüler/innen dürfen nur dann ins Case Managagement aufgenommen werden, wenn sie sich im letzten Schulbesuchsjahr befinden und eine Übergangsprognose vorliegt.');
INSERT INTO tags (id, label, description) VALUES (103, 'CM nur startbar wenn letzte Jahr und Prognose', 'Schüler/innen dürfen nur dann ins Case Managagement aufgenommen werden, wenn sie sich im letzten Schulbesuchsjahr befinden und eine Übergangsprognose vorliegt.');
INSERT INTO tags (id, label, description) VALUES (104, 'CM nur startbar wenn letzte Jahr und Prognose', 'Schüler/innen dürfen nur dann ins Case Managagement aufgenommen werden, wenn sie sich im letzten Schulbesuchsjahr befinden und eine Übergangsprognose vorliegt.');
INSERT INTO tags (id, label, description) VALUES (105, 'Das Geburtsdatum darf nicht in der Zukunft liegen', 'Das Datum darf nicht in der Zukunft liegen');
INSERT INTO tags (id, label, description) VALUES (106, 'cm nur startbar wenn notwendigkeit besteht', 'Die Angabe steht im Widerspruch zur aktuell laufenden Phase');
INSERT INTO tags (id, label, description) VALUES (107, 'cm nur startbar wenn notwendigkeit besteht', 'Die Angabe steht im Widerspruch zur aktuell laufenden Phase');
INSERT INTO tags (id, label, description) VALUES (108, 'cm nur startbar wenn notwendigkeit besteht', 'Die Angabe steht im Widerspruch zur aktuell laufenden Phase');
INSERT INTO tags (id, label, description) VALUES (109, 'cm nur startbar wenn notwendigkeit besteht', 'Die Angabe steht im Widerspruch zur aktuell laufenden Phase');
INSERT INTO tags (id, label, description) VALUES (110, 'ber nur startbar wenn keine notwendigkeit besteht', 'Die Angabe steht im Widerspruch zur aktuell laufenden Phase');
INSERT INTO tags (id, label, description) VALUES (111, 'ber nur startbar wenn keine notwendigkeit besteht', 'Die Angabe steht im Widerspruch zur aktuell laufenden Phase');
INSERT INTO tags (id, label, description) VALUES (112, 'Rückdatierung von CM-Fällen verhindern', 'Das Datum darf nicht vor dem 1.9.2010 liegen');
INSERT INTO tags (id, label, description) VALUES (113, 'phase ea nicht länger als 36 monate', 'Die eingetragenen Datumsangaben sind offensichtlich falsch und überschreiten die maximale Dauer der Phase "Eingangsanamnese". Bitte korrigieren Sie Ihre Angaben, da die Fallakte andernfalls nicht auswertbar ist.');
INSERT INTO tags (id, label, description) VALUES (114, 'phase ber nicht länger als 60 monate', 'Die eingetragenen Datumsangaben sind offensichtlich falsch und überschreiten die maximale Dauer der Phase "Beratung". Bitte korrigieren Sie Ihre Angaben, da die Fallakte andernfalls nicht auswertbar ist.');
INSERT INTO tags (id, label, description) VALUES (115, 'Rückdatierung von CM-Fällen verhindern', 'Das Datum darf nicht vor dem 1.9.2010 liegen');
INSERT INTO tags (id, label, description) VALUES (116, 'phase cm nicht länger als 60 monate', 'Die eingetragenen Datumsangaben sind offensichtlich falsch und überschreiten die maximale Dauer der Phase "Case Management". Bitte korrigieren Sie Ihre Angaben, da die Fallakte andernfalls nicht auswertbar ist.');
INSERT INTO tags (id, label, description) VALUES (117, 'Nachbetreuung nur startbar wenn Art der Beendigung nicht Abbruch ist', 'Die Angabe steht im Widerspruch zur aktuell laufenden Phase');
INSERT INTO tags (id, label, description) VALUES (118, 'Nachbetreuung nur startbar wenn Art der Beendigung nicht Abbruch ist', 'Die Angabe steht im Widerspruch zur aktuell laufenden Phase');
INSERT INTO tags (id, label, description) VALUES (119, 'phase nb nicht länger als 36 monate', 'Die eingetragenen Datumsangaben sind offensichtlich falsch und überschreiten die maximale Dauer der Phase "Nachbetreuung". Bitte korrigieren Sie Ihre Angaben, da die Fallakte andernfalls nicht auswertbar ist.');
INSERT INTO tags (id, label, description) VALUES (1000, 'Mindestdatum nicht gegeben', 'Das Datum muss nach dem 1.1.1970 liegen');
INSERT INTO tags (id, label, description) VALUES (1005, 'Das Datum darf nicht "unbekannt" sein.', 'Das Datum darf nicht "unbekannt" sein.');
INSERT INTO tags (id, label, description) VALUES (1006, '"erstgespraech" muss vor "beendigung_nachbetreuung" liegen.', '"Datum des Erstgespräches mit dem Jugendlichen" darf zeitlich nicht nach "Datum des Ende der Nachbetreuung" liegen.');
INSERT INTO tags (id, label, description) VALUES (1007, '"erstgespraech" muss vor "datum_feststellung" liegen.', '"Datum des Erstgespräches mit dem Jugendlichen" darf zeitlich nicht nach "Datum der Entscheidung" liegen.');
INSERT INTO tags (id, label, description) VALUES (1008, '"geburtsdatum" muss vor "erstgespraech" liegen.', '"Geburtsdatum" darf zeitlich nicht nach "Datum des Erstgespräches mit dem Jugendlichen" liegen.');
INSERT INTO tags (id, label, description) VALUES (1009, '"erstgespraech" muss vor "datum_ende_beratung" liegen.', '"Datum des Erstgespräches mit dem Jugendlichen" darf zeitlich nicht nach "Datum Beratungsende" liegen.');
INSERT INTO tags (id, label, description) VALUES (1010, '"erstgespraech" muss vor "beendigung_5" liegen.', '"Datum des Erstgespräches mit dem Jugendlichen" darf zeitlich nicht nach "Beendigung CM (Datum)" liegen.');
INSERT INTO tags (id, label, description) VALUES (1011, '"erstgespraech" darf nicht in der Zukunft liegen.', '"Datum des Erstgespräches mit dem Jugendlichen" darf nicht in der Zukunft liegen.');
INSERT INTO tags (id, label, description) VALUES (1013, '"geburtsdatum" muss vor "beendigung_5" liegen.', '"Geburtsdatum" darf zeitlich nicht nach "Beendigung CM (Datum)" liegen.');
INSERT INTO tags (id, label, description) VALUES (1014, '"geburtsdatum" muss vor "datum_feststellung" liegen.', '"Geburtsdatum" darf zeitlich nicht nach "Datum der Entscheidung" liegen.');
INSERT INTO tags (id, label, description) VALUES (1015, '"geburtsdatum" muss vor "beendigung_nachbetreuung" liegen.', '"Geburtsdatum" darf zeitlich nicht nach "Datum des Ende der Nachbetreuung" liegen.');
INSERT INTO tags (id, label, description) VALUES (1016, '"geburtsdatum" muss vor "datum_ende_beratung" liegen.', '"Geburtsdatum" darf zeitlich nicht nach "Datum Beratungsende" liegen.');
INSERT INTO tags (id, label, description) VALUES (1017, '"geburtsdatum" darf nicht in der Zukunft liegen.', '"Geburtsdatum" darf nicht in der Zukunft liegen.');
INSERT INTO tags (id, label, description) VALUES (1025, 'Das Datum darf nicht "unbekannt" sein.', 'Das Datum darf nicht "unbekannt" sein.');
INSERT INTO tags (id, label, description) VALUES (1026, '"datum_feststellung" muss vor "beendigung_5" liegen.', '"Datum der Entscheidung" darf zeitlich nicht nach "Beendigung CM (Datum)" liegen.');
INSERT INTO tags (id, label, description) VALUES (1027, '"datum_feststellung" muss vor "datum_ende_beratung" liegen.', '"Datum der Entscheidung" darf zeitlich nicht nach "Datum Beratungsende" liegen.');
INSERT INTO tags (id, label, description) VALUES (1028, '"datum_feststellung" muss vor "beendigung_nachbetreuung" liegen.', '"Datum der Entscheidung" darf zeitlich nicht nach "Datum des Ende der Nachbetreuung" liegen.');
INSERT INTO tags (id, label, description) VALUES (1029, '"datum_feststellung" darf nicht in der Zukunft liegen.', '"Datum der Entscheidung" darf nicht in der Zukunft liegen.');
INSERT INTO tags (id, label, description) VALUES (1032, 'Das Datum darf nicht "unbekannt" sein.', 'Das Datum darf nicht "unbekannt" sein.');
INSERT INTO tags (id, label, description) VALUES (1033, '"datum_ende_beratung" darf nicht in der Zukunft liegen.', '"Datum Beratungsende" darf nicht in der Zukunft liegen.');
INSERT INTO tags (id, label, description) VALUES (1035, 'Das Datum darf nicht "unbekannt" sein.', 'Das Datum darf nicht "unbekannt" sein.');
INSERT INTO tags (id, label, description) VALUES (1036, '"beendigung_5" muss vor "beendigung_nachbetreuung" liegen.', '"Beendigung CM (Datum)" darf zeitlich nicht nach "Datum des Ende der Nachbetreuung" liegen.');
INSERT INTO tags (id, label, description) VALUES (1037, '"beendigung_5" darf nicht in der Zukunft liegen.', '"Beendigung CM (Datum)" darf nicht in der Zukunft liegen.');
INSERT INTO tags (id, label, description) VALUES (1042, 'Das Datum darf nicht "unbekannt" sein.', 'Das Datum darf nicht "unbekannt" sein.');
INSERT INTO tags (id, label, description) VALUES (1043, '"beendigung_nachbetreuung" darf nicht in der Zukunft liegen.', '"Datum des Ende der Nachbetreuung" darf nicht in der Zukunft liegen.');

-- Move tags serial: userdefined tags start at 1000
ALTER SEQUENCE tags_id_seq RESTART 10000;

CREATE TABLE dbinfo (
    id INTEGER PRIMARY KEY NOT NULL,
    key   VARCHAR(128) NOT NULL,
    value VARCHAR(256) 
);

INSERT INTO dbinfo (id, key, value) VALUES (1, 'version', CURRENT_DATE);
INSERT INTO dbinfo (id, key, value) VALUES (2, 'datadump', NULL);

GRANT SELECT ON dbinfo TO PUBLIC;
