--
-- $Revision: 6415e4e151ce $ 
--

BEGIN;

-- CREATE PROCEDURAL LANGUAGE plpgsql;

SET search_path = public, pg_catalog;

-- PL/Python ist eine 'untrusted' Sprache
SET ROLE postgres;

-- Generiert eine UUID
--
CREATE OR REPLACE FUNCTION generate_uuidgen()
RETURNS   text AS
$BODY$
    from commands import getstatusoutput
    from random   import randint
    try:
        result = getstatusoutput("/usr/bin/uuid -v4")
        if result[0] != 0: raise Exception()
        return result[1].strip()
    except:
        count = 500
        while count > 0:
            nid = "%08x-%04x-%04x-%04x-%012x" % (
                randint(0, 1 <<  8*4),
                randint(0, 1 <<  4*4),
                randint(0, 1 <<  4*4),
                randint(0, 1 <<  4*4),
                randint(0, 1 << 12*4))
            if len(nid) == 36:
                return nid
            count -= 1
        raise Exception("UUID generator failed")
$BODY$
LANGUAGE 'plpythonu' VOLATILE;

REVOKE ALL ON FUNCTION generate_uuidgen() FROM public;
GRANT EXECUTE ON FUNCTION generate_uuidgen() TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION generate_uuidgen() TO GROUP :admin_group;
ALTER FUNCTION generate_uuidgen() OWNER TO :adm_ka_owner;

-- Ab hier als :adm_ka_owner

SET ROLE :adm_ka_owner;

--
-- Setzt in der Tabelle 't_name' in der Spalte mit der
-- ID 'id' das Feld 'uuid_id' mit einer UUID.
-- Bei Verletzung des Unique-Constraints wird bis
-- zu 500 mal neu gewuerfelt.
--
CREATE OR REPLACE FUNCTION set_uuid(t_name text, id int) 
RETURNS void AS 
$$
DECLARE
    xuuid_id char(36);
BEGIN
    FOR i IN 0..500 LOOP
        SELECT INTO xuuid_id generate_uuidgen();
        BEGIN
            EXECUTE 'UPDATE '           || t_name   || 
                    ' SET uuid_id = ''' || xuuid_id ||
                    ''' WHERE id = '    || id;
        EXCEPTION WHEN unique_violation THEN
            CONTINUE;
        END;
        RETURN;
    END LOOP;
    RAISE EXCEPTION 'set_uuid failed';
END;
$$ LANGUAGE plpgsql;
REVOKE ALL ON FUNCTION set_uuid(text, int) FROM public;

--
-- Funktionen zum Anlegen neuer (Teile von) Fallakten
-- Diese sind nur fuer die Gruppe CM_KA aufrufbar (Admin_KA darf keine neuen 
-- Akten anlegen) und laufen mit den Berechtigungen des zentralen
-- administrativen Benutzers der KA
CREATE OR REPLACE FUNCTION create_master_ds(nuuid char(36)) 
    RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS 
$$
DECLARE
    standin integer;
    mid     integer;
BEGIN
    INSERT INTO master_tbl (bearbeiter_id) VALUES (getuserid());
    SELECT INTO standin vertreter from ka_benutzer_tbl WHERE id = getuserid();

    IF standin IS NOT NULL THEN 
        INSERT INTO nm_benutzer_master_tbl(benutzer_id, master_id) 
                VALUES (standin, currval('master_tbl_id_seq'));
    END IF;

    SELECT INTO mid currval('master_tbl_id_seq'); 

    IF nuuid IS NOT NULL THEN
        UPDATE master_tbl SET uuid_id = nuuid WHERE id = mid;
    ELSE
        PERFORM set_uuid('master_tbl', mid);
    END IF;

    PERFORM create_status_ds(mid);
    RETURN mid;
END;
$$;

REVOKE ALL ON FUNCTION create_master_ds(char(36)) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_master_ds(char(36))
    TO GROUP :cm_group;

--
-- Funktion, die ueberprueft, ob eine Kundennummer schon existiert.
--
CREATE OR REPLACE FUNCTION case_exists(knummer varchar(100)) 
    RETURNS boolean
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
    found boolean;
BEGIN
    SELECT INTO found TRUE FROM master_tbl WHERE kundennummer = knummer;
    IF found THEN
        RETURN TRUE;
    ELSE
        RETURN FALSE;
    END IF;
END;
$$ ;

REVOKE ALL ON FUNCTION case_exists(varchar) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION case_exists(varchar(100)) TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION case_exists(varchar(100)) TO GROUP :cm_group;

--
-- Funktionen zum Loeschen einer Fallakte inklusive aller angehaengten 
-- Unterstuetzungsangebote, Dateien Termine etc.
-- Diese sind nur fuer die Gruppe ADMIN_KA aufrufbar (CM_KA sollen keine Akten
-- loeschen koennen) und laufen mit den Berechtigungen des zentralen
-- administrativen Benutzers der KA

CREATE OR REPLACE FUNCTION delete_master_ds(mid integer) RETURNS void 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
    st integer;
BEGIN
    SELECT INTO st s.status FROM master_tbl m JOIN ka_status_tbl s
        ON s.master_id = m.id WHERE m.id = mid;

    IF NOT st in (3, 5) THEN
        RAISE EXCEPTION 'case is not deletable';
    END IF;

    DELETE FROM master_tbl WHERE id = mid;
END;
$$ ;

REVOKE ALL    ON FUNCTION delete_master_ds(integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION delete_master_ds(integer) TO GROUP :admin_group;

--
-- Wenn eine Akte anonymisiert ist, hat sie keine UUID mehr. Folglich
-- Kann man aus der anonymized_uuids auch nichts mehr loeschen.
-- Falls sich dort trotzdem ein gleicher Eintrag finden sollte, ist
-- dies eine potentielle Quelle fuer eine Inkonsistenz. 
--
CREATE OR REPLACE FUNCTION delete_anonymized_uuid() RETURNS TRIGGER AS $$
BEGIN
    -- Diese _sollte_ nichts loeschen!
    DELETE FROM anonymized_uuids WHERE uuid_id = OLD.uuid_id;
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER t_delete_anonymized_uuid 
    AFTER DELETE ON master_tbl
    FOR EACH ROW EXECUTE PROCEDURE delete_anonymized_uuid();

--
-- Unterstuetzungsangebote im Bereich der Berufsvorbereitung
--

CREATE OR REPLACE FUNCTION create_rg_beruf_ds(mid integer, nuuid char(36)) 
RETURNS integer
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS 
$$
DECLARE
    rid integer;
BEGIN
    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    INSERT INTO rg_angebote_berufsvorbereitung_tbl(master_id) VALUES (mid);
    SELECT INTO rid currval('rg_angebote_berufsvorbereitung_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE rg_angebote_berufsvorbereitung_tbl 
        SET    uuid_id = nuuid 
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('rg_angebote_berufsvorbereitung_tbl', rid);
    END IF;

    RETURN rid;
END;
$$ ;

REVOKE ALL ON FUNCTION create_rg_beruf_ds(integer, char(36)) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_rg_beruf_ds(integer, char(36)) 
    TO GROUP :cm_group;


CREATE OR REPLACE FUNCTION delete_rg_beruf_ds(dsid integer) RETURNS void 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid master_id FROM rg_angebote_berufsvorbereitung_tbl 
            WHERE id = dsid;
    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;
    DELETE FROM rg_angebote_berufsvorbereitung_tbl WHERE id = dsid;
END;
$$ ;

REVOKE ALL ON FUNCTION delete_rg_beruf_ds(dsid integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION delete_rg_beruf_ds(integer) TO GROUP :cm_group;


--
-- Unterstuetzungsangebote im allgemein bildenden Bereich
--

CREATE OR REPLACE FUNCTION create_rg_bilden_ds(mid integer, nuuid char(36)) 
RETURNS integer
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS 
$$
DECLARE
    rid integer;
BEGIN
    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    INSERT INTO rg_angebote_bildenden_bereich_tbl(master_id) VALUES (mid);
    SELECT INTO rid currval('rg_angebote_bildenden_bereich_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE rg_angebote_bildenden_bereich_tbl 
        SET    uuid_id = nuuid 
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('rg_angebote_bildenden_bereich_tbl', rid);
    END IF;

    RETURN rid;
END;
$$ ;

REVOKE ALL ON FUNCTION create_rg_bilden_ds(integer, char(36)) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_rg_bilden_ds(integer, char(36))
    TO GROUP :cm_group;


CREATE OR REPLACE FUNCTION delete_rg_bilden_ds(dsid integer) RETURNS void 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid master_id FROM rg_angebote_bildenden_bereich_tbl 
            WHERE id = dsid;
    IF not isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;
    DELETE FROM rg_angebote_bildenden_bereich_tbl WHERE id = dsid;
END;
$$ ;

REVOKE ALL ON FUNCTION delete_rg_bilden_ds(integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION delete_rg_bilden_ds(integer) TO GROUP :cm_group;


--
-- Unterstuetzungsangebote im Bereich der Lebensbewaeltigung
--

CREATE OR REPLACE FUNCTION create_rg_leben_ds(mid integer, nuuid char(36)) 
RETURNS integer
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$
DECLARE
    rid integer;
BEGIN
    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;
    INSERT INTO rg_angebote_lebensbewaeltigung_tbl(master_id) VALUES (mid);
    SELECT INTO rid currval('rg_angebote_lebensbewaeltigung_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE rg_angebote_lebensbewaeltigung_tbl 
        SET    uuid_id = nuuid 
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('rg_angebote_lebensbewaeltigung_tbl', rid);
    END IF;

    RETURN rid;
END;
$$ ;

REVOKE ALL ON FUNCTION create_rg_leben_ds(integer, char(36)) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_rg_leben_ds(integer, char(36))
    TO GROUP :cm_group;


CREATE OR REPLACE FUNCTION delete_rg_leben_ds(dsid integer) RETURNS void 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid master_id FROM rg_angebote_lebensbewaeltigung_tbl 
            WHERE id = dsid;
    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;
    DELETE FROM rg_angebote_lebensbewaeltigung_tbl WHERE id = dsid;
END;
$$ ;

REVOKE ALL ON FUNCTION delete_rg_leben_ds(integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION delete_rg_leben_ds(integer) TO GROUP :cm_group;

--
-- Unterstuetzungsangebote im Bereich der beruflichen Qualifizierung
--

CREATE OR REPLACE FUNCTION create_rg_quali_ds(mid integer, nuuid char(36)) 
RETURNS integer
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$
DECLARE
    rid integer;
BEGIN
    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;
    INSERT INTO rg_angebote_berufliche_qualifizierung_tbl (master_id) 
            VALUES (mid);
    SELECT INTO rid currval('rg_angebote_berufliche_qualifizierung_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE rg_angebote_berufliche_qualifizierung_tbl 
        SET    uuid_id = nuuid 
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('rg_angebote_berufliche_qualifizierung_tbl', rid);
    END IF;

    RETURN rid;
END;
$$ ;

REVOKE ALL ON FUNCTION create_rg_quali_ds(integer, char(36)) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_rg_quali_ds(integer, char(36))
    TO GROUP :cm_group;

CREATE OR REPLACE FUNCTION delete_rg_quali_ds(dsid integer) RETURNS void 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid master_id FROM rg_angebote_berufliche_qualifizierung_tbl 
            WHERE id = dsid;
    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;
    DELETE FROM rg_angebote_berufliche_qualifizierung_tbl WHERE id = dsid;
END;
$$ ;

REVOKE ALL ON FUNCTION delete_rg_quali_ds(integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION delete_rg_quali_ds(integer) TO GROUP :cm_group;


--
-- Massnahmen 
--
CREATE OR REPLACE FUNCTION create_rg_massnahme_ds(mid integer) RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;
    INSERT INTO rg_massnahme_tbl(master_id) VALUES (mid);
    RETURN currval('rg_massnahme_tbl_id_seq');
END;
$$ ;

REVOKE ALL ON FUNCTION create_rg_massnahme_ds(integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_rg_massnahme_ds(integer) 
    TO GROUP :cm_group;


--
-- Kompetenzfeststellung
--

CREATE OR REPLACE FUNCTION create_rg_kompetenz_ds(mid integer, nuuid char(36)) 
    RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
    rid integer;
BEGIN
    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    INSERT INTO rg_kompetenzfestellung_tbl(master_id) VALUES (mid);
    SELECT INTO rid currval('rg_kompetenzfestellung_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE rg_kompetenzfestellung_tbl 
        SET    uuid_id = nuuid 
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('rg_kompetenzfestellung_tbl', rid);
    END IF;

    RETURN rid;
END;
$$ ;

REVOKE ALL ON FUNCTION create_rg_kompetenz_ds(integer, char(36)) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_rg_kompetenz_ds(integer, char(36))
    TO GROUP :cm_group;


--
-- Sprachen
--
CREATE OR REPLACE FUNCTION create_rg_sprache_ds(rgid integer) RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
        mid integer;
BEGIN
    SELECT INTO mid master_id FROM rg_kompetenzfestellung_tbl WHERE id = rgid;
        IF NOT isEditorOrStandin(mid) THEN                 
        RAISE EXCEPTION 'DS ID not session_user ID';         
    END IF;         
    INSERT INTO rg_beherrschung_sprache_tbl(rg_kompetenzfestellung_id) 
        VALUES (rgid);
    RETURN currval('rg_beherrschung_sprache_tbl_id_seq');

END;
$$ ;

REVOKE ALL ON FUNCTION create_rg_sprache_ds(integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_rg_sprache_ds( integer ) 
    TO GROUP :cm_group;


--
-- Funktionen zum Erstellen und Loeschen von Konfigurationen 
--

CREATE OR REPLACE FUNCTION create_konfiguration_ds(_bez varchar, _wert varchar) RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
    INSERT INTO ka_konfiguration_tbl(bez, wert) VALUES (_bez, _wert);
    RETURN currval('ka_konfiguration_tbl_id_seq');
END;
$$ ;

CREATE OR REPLACE FUNCTION delete_konfiguration_ds(kid integer ) RETURNS void 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
    DELETE FROM ka_konfiguration_tbl WHERE id = kid;
END;
$$ ;


--
-- Funktionen zum Erstellen von Dokumenten
--
CREATE OR REPLACE FUNCTION create_fall_dokument_ds(mid integer, nuuid char(36), filename varchar) RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE 
	did integer;
	rid integer;
BEGIN
    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    SELECT INTO did id from ka_dokumente_tbl WHERE name = filename AND mid = master_id;
    IF NOT FOUND THEN
    	INSERT INTO ka_dokumente_tbl(master_id) VALUES (mid);
    	SELECT INTO rid currval('ka_dokumente_tbl_id_seq');
    ELSE 
        RETURN did;
    END IF;

    IF nuuid IS NOT NULL THEN
        UPDATE ka_dokumente_tbl 
        SET    uuid_id = nuuid 
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('ka_dokumente_tbl', rid);
    END IF;
    RETURN rid;
END;
$$ ;

REVOKE ALL ON FUNCTION create_fall_dokument_ds(integer, char(36), varchar) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_fall_dokument_ds( integer, char(36), varchar) TO GROUP :cm_group;


CREATE OR REPLACE FUNCTION delete_fall_dokument_ds(did integer) RETURNS void 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid master_id FROM ka_dokumente_tbl WHERE id = did;
    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;
    DELETE FROM ka_dokumente_tbl WHERE id = did;
END;
$$ ;

REVOKE ALL ON FUNCTION delete_fall_dokument_ds(integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION delete_fall_dokument_ds(integer) 
                TO GROUP :cm_group;

--
-- Funktion, die alle Anhaenge zu einem gegebenen Fall loescht
--
CREATE OR REPLACE FUNCTION delete_all_fall_dokumente(mid integer) 
RETURNS void 
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
AS $$
BEGIN
    DELETE FROM ka_dokumente_tbl WHERE master_id = mid;
END;
$$;

REVOKE ALL     ON FUNCTION delete_all_fall_dokumente(integer) FROM PUBLIC;
GRANT  EXECUTE ON FUNCTION delete_all_fall_dokumente(integer) TO GROUP :admin_group;

-- Globale Dokumente 

CREATE OR REPLACE FUNCTION create_global_dokument_ds(filename varchar) RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
    did integer;
BEGIN
    SELECT INTO did id from ka_dokumente_tbl WHERE name = filename and master_id IS NULL;
    IF  NOT FOUND THEN
    	INSERT INTO ka_dokumente_tbl (master_id) VALUES (NULL);
    	RETURN currval('ka_dokumente_tbl_id_seq');
    ELSE
        RETURN did;
    END IF;
END;
$$ ;

REVOKE ALL ON FUNCTION create_global_dokument_ds(varchar) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_global_dokument_ds(varchar) TO GROUP :admin_group;


CREATE OR REPLACE FUNCTION delete_global_dokument_ds(did integer) RETURNS void 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
    DELETE FROM ka_dokumente_tbl WHERE id = did;
END;
$$ ;

REVOKE ALL ON FUNCTION delete_global_dokument_ds(did integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION delete_global_dokument_ds( integer ) 
            TO GROUP :admin_group;

--
-- Funktionen zum Erstellen und Löschen von Terminen
--

CREATE OR REPLACE FUNCTION create_fall_termin_ds(mid integer) RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;
    INSERT INTO ka_termine_tbl(master_id) VALUES (mid);
    RETURN currval('ka_termine_tbl_id_seq');
END;
$$ ;

REVOKE ALL ON FUNCTION create_fall_termin_ds(integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_fall_termin_ds( integer ) 
                TO GROUP :cm_group;


CREATE OR REPLACE FUNCTION delete_fall_termin_ds(tid integer) RETURNS void 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid master_id FROM ka_termine_tbl WHERE id = tid;
    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;
    DELETE FROM ka_termine_tbl WHERE id = tid;
END;
$$ ;

REVOKE ALL ON FUNCTION delete_fall_termin_ds(integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION delete_fall_termin_ds(integer) 
                TO GROUP :cm_group;

--
-- Funktion, die alle Termine zu einem gegebenen Fall loescht
--
CREATE OR REPLACE FUNCTION delete_all_fall_termine(mid integer) 
RETURNS void 
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
AS $$
BEGIN
    DELETE FROM ka_termine_tbl WHERE master_id = mid;
END;
$$;

REVOKE ALL     ON FUNCTION delete_all_fall_termine(integer) FROM PUBLIC;
GRANT  EXECUTE ON FUNCTION delete_all_fall_termine(integer) TO GROUP :admin_group;

-- Globale Dokumente 

CREATE OR REPLACE FUNCTION create_global_termin_ds() RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
    INSERT INTO ka_termine_tbl (master_id) VALUES (NULL);
    RETURN currval('ka_termine_tbl_id_seq');
END;
$$ ;

REVOKE ALL ON FUNCTION create_global_termin_ds() FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_global_termin_ds() TO GROUP :admin_group;


CREATE OR REPLACE FUNCTION delete_global_termin_ds(tid integer) RETURNS void 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
    DELETE FROM ka_termine_tbl WHERE id = tid;
END;
$$ ;

REVOKE ALL ON FUNCTION delete_global_termin_ds(tid integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION delete_global_termin_ds( integer ) 
            TO GROUP :admin_group;

--
-- Dies Funktion 'delete_old_anonymized_cases' loescht alle anonymisierten 
-- Fallakten, die laenger als die Aufbewahrungsfrist der jeweiligen KA
-- unbearbeitet im System liegen.
--
CREATE OR REPLACE FUNCTION delete_old_anonymized_cases()
RETURNS void AS
$$
DECLARE
    mid          int;
    max_age      interval;
    max_age_text text;
BEGIN
    SELECT INTO max_age_text wert FROM ka_konfiguration_tbl_view WHERE bez = 'max_speicherdauer';
    IF NOT FOUND
    THEN
        RETURN;
    END IF;

    max_age = (max_age_text || ' day')::interval;

    FOR mid IN SELECT m.id FROM master_tbl_view m
               JOIN ka_status_tbl_view s
               ON m.id = s.master_id
               WHERE
               s.status = 5 AND age(s.zugriff) >= max_age
    LOOP
        PERFORM delete_master_ds(mid);
    END LOOP;
END;
$$ LANGUAGE plpgsql;

REVOKE ALL    ON FUNCTION delete_old_anonymized_cases() FROM public;
GRANT EXECUTE ON FUNCTION delete_old_anonymized_cases() TO GROUP :admin_group;

-- ----------------------------------------------------------------------------
-- 
-- Haeufige Abfragen, die sowohl von der Gruppe CM_KA, als auch Admin_KA
-- ausgefuehrt werden koennen.
-- 

--
-- gettotal()
-- Anzahl aller Faelle des Fallmanagers oder der KA (wenn Admin)
--
CREATE OR REPLACE FUNCTION gettotal() RETURNS integer
    LANGUAGE 'plpgsql' 
    AS $$
DECLARE 
    total int4;
BEGIN
    SELECT INTO total count(id) FROM master_tbl_eval_total_view;
    RETURN total;
END;
$$ ;

REVOKE ALL ON FUNCTION gettotal() FROM PUBLIC;
GRANT EXECUTE ON FUNCTION gettotal() TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION gettotal() TO GROUP :admin_group;


--
-- gettotalcm()
-- Anzahl aller Faelle des Fallmanagers oder der KA (wenn Admin),
-- die sich im Case-Management befinden.
--
CREATE OR REPLACE FUNCTION gettotalcm() RETURNS integer
    LANGUAGE 'plpgsql' 
    AS $$
DECLARE
    total_cm int4;
BEGIN
    SELECT INTO total_cm count(id) FROM master_tbl_eval_incm_view;
    RETURN total_cm;
END;
$$ ;

REVOKE ALL ON FUNCTION gettotalcm() FROM PUBLIC;
GRANT EXECUTE ON FUNCTION gettotalcm() TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION gettotalcm() TO GROUP :admin_group;

--
-- Definition von DB-seitigen Funktionen zur Nutzerverwaltung
-- 
-- Autor: Frank Koormann <frank@intevation.de>
-- Stand: 2007-11-01
--

-- ------------------------------------------------------------------------
-- Anlegen einer Rolle in der DB einer KA:
-- - Ueber mpuls_create_role() Datenbank-Rolle anlegen und 
-- - dann Eintragungen in ka_benutzer_tbl machen.
CREATE OR REPLACE FUNCTION ka_create_role (
        agency          varchar(52),
        grouprole       varchar(64),
        loginname       varchar(64),
        firstname       varchar(64),
        surname         varchar(64),
        phone           varchar(64),
        room            varchar(64),
        branchoffice    varchar(64),
        activated       boolean,
        setpassword     boolean
    ) RETURNS varchar(9) 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
    newrole    RECORD;
    groupid    integer;
    userid     integer;
BEGIN
    SELECT INTO newrole * FROM mpuls_create_role(loginname, grouprole, agency)
        AS (db_name varchar(64), db_user varchar(64), pw varchar(9));
    EXECUTE 'GRANT CONNECT ON DATABASE ' || newrole.db_name 
                || ' TO ' || newrole.db_user; 
    INSERT INTO ka_benutzer_tbl ( login, vorname, nachname,
        telefon, raumnummer, filiale,
        aktiviert, passwort_aendern, gid) 
        VALUES
        (newrole.db_user, firstname, surname,
        phone, room, branchoffice,
        activated, setpassword, NULL);
    SELECT INTO userid currval('ka_benutzer_tbl_id_seq');
    SELECT INTO groupid create_benutzergruppe_ds(surname ||', '|| firstname ||' ('||loginname||')');
    UPDATE ka_benutzer_tbl SET gid = groupid WHERE id = userid;
    INSERT INTO nm_benutzer_gruppe_tbl (benutzer_id, gruppe_id) VALUES (userid, groupid);

    RETURN newrole.pw::varchar(9);
END;
$$ ;

REVOKE ALL ON FUNCTION ka_create_role(varchar, varchar, varchar, 
    varchar, varchar, varchar, varchar, varchar, boolean, boolean) 
        FROM PUBLIC;
GRANT EXECUTE ON FUNCTION ka_create_role(varchar, varchar, varchar, 
    varchar, varchar, varchar, varchar, varchar, boolean, boolean) 
        TO GROUP :admin_group;

CREATE OR REPLACE FUNCTION ka_delete_role (
        loginname       varchar(64),
        agency          varchar(52)
    ) RETURNS VOID 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
    db_user  varchar(128);
    groupid   integer;
BEGIN
    db_user := quote_ident('ka_' || agency || '_' || loginname);
    SELECT INTO groupid gid FROM ka_benutzer_tbl WHERE login = db_user;
    DELETE FROM ka_benutzer_tbl WHERE login = db_user;
    DELETE FROM ka_benutzergruppe_tbl WHERE id = groupid;
    PERFORM mpuls_drop_role(loginname, agency);
END;
$$ ;

REVOKE ALL ON FUNCTION ka_delete_role(varchar, varchar) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION ka_delete_role(varchar, varchar) 
            TO GROUP :admin_group;


-- Funktionen zu Erstellen und Loeschen neuer Eintraeger in der
-- Benutzergruppen Tabelle

CREATE OR REPLACE FUNCTION create_benutzergruppe_ds(gname varchar) RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
    INSERT INTO ka_benutzergruppe_tbl (name) VALUES (gname);
    RETURN currval('ka_benutzergruppe_tbl_id_seq');
END;
$$ ;

REVOKE ALL ON FUNCTION create_benutzergruppe_ds( varchar ) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_benutzergruppe_ds( varchar ) TO GROUP :admin_group;

CREATE OR REPLACE FUNCTION delete_benutzergruppe_ds(gid integer) RETURNS void 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
BEGIN
    DELETE FROM ka_benutzergruppe_tbl WHERE id = gid;
END;
$$ ;

REVOKE ALL ON FUNCTION delete_benutzergruppe_ds( integer ) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION delete_benutzergruppe_ds( integer ) TO GROUP :admin_group;

--
-- Voreinstellung Vertretungsregeln setzen
--
CREATE OR REPLACE FUNCTION ka_set_standin (standin_id integer, user_id integer)
    RETURNS VOID 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
    UPDATE ka_benutzer_tbl_view set vertreter = standin_id WHERE id = user_id; 
END;
$$ ;

REVOKE ALL ON FUNCTION ka_set_standin(integer, integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION ka_set_standin ( integer, integer ) TO :cm_group;


--
-- Funktion um die Flagge zurueckzusetzten, dass der Benutzer sein Passwort bei
-- der Neuanmeldung neu setzen muss. Die Funktion muss auch fuer die Benutzer
-- zugaenglich sein.
-- 
CREATE OR REPLACE FUNCTION ka_unset_passrefresh_flag ( userlogin varchar ) 
    RETURNS VOID 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
    UPDATE ka_benutzer_tbl_view set passwort_aendern = False 
            WHERE login = userlogin; 
END;
$$ ;

REVOKE ALL ON FUNCTION ka_unset_passrefresh_flag(varchar) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION ka_unset_passrefresh_flag ( userlogin varchar ) 
                TO :admin_group;
GRANT EXECUTE ON FUNCTION ka_unset_passrefresh_flag ( userlogin varchar ) 
                TO :cm_group;

-- 
-- Funktion zum Setzen des Benutzerpassworts
--

CREATE OR REPLACE FUNCTION ka_set_password(loginname varchar, pw varchar)
    RETURNS text 
    LANGUAGE 'plpgsql'
    AS $$
BEGIN
    EXECUTE 'ALTER USER ' || loginname || ' WITH PASSWORD ' 
              || quote_literal(pw);
    EXECUTE ka_unset_passrefresh_flag(loginname);
          RETURN pw::text;
    RETURN ''::text;
END;
$$ ;

REVOKE ALL ON FUNCTION ka_set_password(varchar, varchar) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION ka_set_password(varchar, varchar) 
                TO :admin_group;
GRANT EXECUTE ON FUNCTION ka_set_password(varchar, varchar)    
                TO :cm_group;


--
-- Funktionen zum Hinzufuegen und Loeschen der Benutzer in die
-- Benutzer-Fallakten Transitionstabelle, sowie zum Setzen des Bearbeiters
-- einer Fallakte.
--

CREATE OR REPLACE FUNCTION create_nm_benutzer_master_ds ( userid integer, 
    masterid integer ) RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
    INSERT INTO nm_benutzer_master_tbl (benutzer_id, master_id) 
        VALUES (userid, masterid);
    RETURN currval('nm_benutzer_master_tbl_id_seq');
END;
$$ ;

REVOKE ALL ON FUNCTION create_nm_benutzer_master_ds(integer, integer) 
                FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_nm_benutzer_master_ds(integer, integer) 
                TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION create_nm_benutzer_master_ds(integer, integer) 
                TO GROUP :admin_group;


CREATE OR REPLACE FUNCTION delete_nm_benutzer_master_ds ( userid integer,
    masterid integer ) RETURNS VOID 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
    DELETE FROM nm_benutzer_master_tbl 
            WHERE master_id = masterid AND benutzer_id = userid; 
END;
$$ ;

REVOKE ALL ON FUNCTION delete_nm_benutzer_master_ds(integer, integer) 
                FROM PUBLIC;
GRANT EXECUTE ON FUNCTION delete_nm_benutzer_master_ds(integer, integer) 
                TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION delete_nm_benutzer_master_ds(integer, integer) 
                TO GROUP :admin_group;

-- Funktionen zu Erstellen und Loeschen neuer Eintraeger in der
-- nm_benutzer_gruppe Tabelle

CREATE OR REPLACE FUNCTION create_nm_benutzer_gruppe_ds(bid integer, gid integer) RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
    INSERT INTO nm_benutzer_gruppe_tbl (benutzer_id, gruppe_id) VALUES (bid, gid);
    RETURN currval('nm_benutzer_gruppe_tbl_id_seq');
END;
$$ ;
REVOKE ALL ON FUNCTION create_nm_benutzer_gruppe_ds( integer, integer ) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_nm_benutzer_gruppe_ds( integer, integer ) TO GROUP :admin_group;

CREATE OR REPLACE FUNCTION delete_nm_benutzer_gruppe_ds( bid integer, gid integer) RETURNS void 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
    DELETE FROM nm_benutzer_gruppe_tbl WHERE gruppe_id = gid AND benutzer_id = bid;
END;
$$ ;
REVOKE ALL ON FUNCTION delete_nm_benutzer_gruppe_ds( integer, integer ) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION delete_nm_benutzer_gruppe_ds( integer, integer ) TO GROUP :admin_group;


CREATE OR REPLACE FUNCTION set_case_editor (userid integer, masterid integer) 
    RETURNS VOID 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
    UPDATE master_tbl set bearbeiter_id = userid WHERE id = masterid;
END;
$$ ;

REVOKE ALL ON FUNCTION set_case_editor(integer, integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION set_case_editor(integer, integer) 
                TO GROUP :admin_group;


CREATE OR REPLACE FUNCTION set_case_editor_name (firstname varchar, lastname varchar, masterid integer) 
    RETURNS VOID 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
    UPDATE master_tbl set mitarb_zv = firstname, mitarb_zn = lastname WHERE id = masterid;
END;
$$ ;

REVOKE ALL ON FUNCTION set_case_editor_name(varchar, varchar, integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION set_case_editor_name(varchar, varchar, integer) 
                TO GROUP :admin_group;

CREATE OR REPLACE FUNCTION getrole( user_id integer ) RETURNS varchar 
    LANGUAGE 'plpgsql' STABLE SECURITY DEFINER 
    AS $$
DECLARE
    role varchar;
    oid integer;
BEGIN
    SELECT p.usesysid INTO oid FROM pg_user p JOIN ka_benutzer_tbl b 
        ON b.login = p.usename 
        WHERE b.id = user_id; 
    SELECT groname INTO role FROM pg_group WHERE oid = ANY (grolist);
    RETURN role;
END;
$$;

REVOKE ALL ON FUNCTION getrole(user_id integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION getrole( integer ) TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION getrole( integer ) TO GROUP :admin_group;

--
-- getuserid()
-- holt id des current_user aus der ka_benutzer_tbl
--
-- Funktion wurde in die 00precond.sql verschoben

--
-- getoid()
-- OID des current_user
--
CREATE OR REPLACE FUNCTION getoid() RETURNS integer
    LANGUAGE 'plpgsql' STABLE
    AS $$
DECLARE
    myoid OID;
BEGIN
    SELECT INTO myoid oid FROM pg_roles WHERE rolname = current_user;
    RETURN myoid;
END;
$$;

REVOKE ALL ON FUNCTION getoid() FROM PUBLIC;
GRANT EXECUTE ON FUNCTION getoid() TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION getoid() TO GROUP :admin_group;

--
-- isEditorOrStandin(integer)
-- Liefert TRUE wenn der Benutzer fuer die Fallakte 
-- a) Bearbeiter oder
-- b) Vertreter ist
-- Sonst FALSE
--

CREATE OR REPLACE FUNCTION isEditorOrStandin(mid integer)
    RETURNS boolean 
    LANGUAGE 'plpgsql' STABLE
    AS $$
DECLARE
    bid integer;
    isAllowed integer;
BEGIN
    SELECT bearbeiter_id into bid from master_tbl_view WHERE id = mid;
    IF bid = getuserid() THEN
        RETURN TRUE;
    ELSE 
        SELECT INTO isAllowed benutzer_id FROM 
        nm_benutzer_master_tbl_view 
        WHERE master_id = mid 
        AND benutzer_id IN (
                SELECT gruppe_id FROM nm_benutzer_gruppe_tbl_view 
                WHERE benutzer_id = getuserid()
                );
    END IF;
    RETURN FOUND;
END;
$$;

REVOKE ALL ON FUNCTION isEditororstandin(integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION isEditorOrStandin(integer) TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION isEditorOrStandin(integer) TO GROUP :admin_group;

--
-- create_status_ds(mid integer)
-- legt einen statusdatensatz fuer eine Fallakte an
-- 
-- deletion of the entry is done by ON DELETE CONSTRAINT

CREATE OR REPLACE FUNCTION create_status_ds(mid integer)
    RETURNS VOID 
    LANGUAGE 'plpgsql' VOLATILE SECURITY DEFINER 
    AS $$
BEGIN
    INSERT INTO ka_status_tbl (master_id, zugriff, status) 
            VALUES (mid, now(), 1);
END;
$$;

REVOKE ALL ON FUNCTION create_status_ds(integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_status_ds(integer) TO GROUP :cm_group;

-- 
-- get_masterid_from_uuid(myrel varchar, uuid char) 
-- Author: Torsten Irlaender 
--
-- Gibt die master id des datensatzes mit der übergebenen uuid in der
-- angegebenen realtion wieder. Wenn ein solcher datensatz nicht existiert, wird
-- NULL zurueck gegeben.
--

CREATE OR REPLACE FUNCTION get_masterid_from_uuid(myrel varchar(128), uuid char(36))
    RETURNS integer AS
$$
DECLARE
    myid integer;
BEGIN
    IF myrel = 'master' THEN
        SELECT INTO myid id from master_tbl WHERE uuid_id = uuid;
    ELSIF myrel = 'logbuch' THEN
        SELECT INTO myid master_id from ka_logbuch_tbl WHERE uuid_id = uuid;
    ELSIF myrel = 'dokumente' THEN
        SELECT INTO myid master_id from ka_dokumente_tbl WHERE uuid_id = uuid;
    END IF;
    RETURN myid;
END;
$$ LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER;
REVOKE ALL ON FUNCTION get_masterid_from_uuid(varchar, char) FROM public;
GRANT EXECUTE ON FUNCTION get_masterid_from_uuid(varchar, char)  TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION get_masterid_from_uuid(varchar, char)  TO GROUP :admin_group;

-- 
-- getLastCaseAppointment(mid int4)
-- 
-- Lieftert das letzte relevante "Datum" zu einem durch den mit
-- "mid" identifiezierten Fall.
-- Die Menge der durchsuchten Termine sind die
-- fallspezifischen Termine sowie das letze Beabreitungsdatum.
--

-- Einfügen wenn auf Unterstützungsangebote durchsucht werden sollen
-- SELECT unterstuetzung_ende as date from
-- rg_angebote_bildenden_bereich_tbl_view
-- WHERE master_id = mid
-- UNION
-- SELECT unterstuetzung_ende_1 as date from
-- rg_angebote_berufsvorbereitung_tbl_view 
-- WHERE master_id = mid
-- UNION
-- SELECT unterstuetzung_ende_2 as date from 
-- rg_angebote_berufliche_qualifizierung_tbl_view 
-- WHERE master_id = mid
-- UNION
-- SELECT unterstuetzung_ende_3 as date from 
-- rg_angebote_lebensbewaeltigung_tbl_view
-- WHERE master_id = mid
-- UNION
CREATE OR REPLACE FUNCTION getlastcaseappointment(mid int4)
	RETURNS "timestamp" AS
$BODY$
DECLARE
	maxdate timestamp;
BEGIN
	SELECT INTO maxdate max(date::timestamp) FROM (
	SELECT zugriff as date from 
	ka_status_tbl_view
	WHERE master_id = mid
	UNION
	SELECT max(sdatum) as date from 
	ka_fall_termine_tbl_view
	WHERE master_id = mid
	UNION
	SELECT max(edatum) as date from 
	ka_fall_termine_tbl_view
	WHERE master_id = mid
	) as test;
	RETURN maxdate;
END;
$BODY$ LANGUAGE 'plpgsql' VOLATILE;
REVOKE ALL ON FUNCTION getlastcaseappointment(int) FROM public;
GRANT EXECUTE ON FUNCTION getlastcaseappointment(int)  TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION getlastcaseappointment(int)  TO GROUP :admin_group;

--
-- Funktionen zum Anlegen und Loeschen von Logbucheintraegen
--
CREATE OR REPLACE FUNCTION create_logbuch_ds(mid integer, nuuid CHAR(36)) RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
AS $$
DECLARE
    lid integer;
    editor_name varchar;
BEGIN
    IF NOT (isEditorOrStandin(mid) OR pg_has_role('admin','MEMBER')) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    SELECT INTO editor_name nachname || ', ' || vorname from ka_benutzer_tbl WHERE id = getuserid();
    INSERT INTO ka_logbuch_tbl(master_id, datum, bearbeiter) VALUES (mid, now(), editor_name);

    SELECT INTO lid currval('ka_logbuch_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE ka_logbuch_tbl SET uuid_id = nuuid WHERE id = lid;
    ELSE
        PERFORM set_uuid('ka_logbuch_tbl', lid);
    END IF;

    RETURN lid;
END;
$$;

REVOKE ALL    ON FUNCTION create_logbuch_ds(integer, CHAR(36)) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_logbuch_ds(integer, CHAR(36)) TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION create_logbuch_ds(integer, CHAR(36)) TO GROUP :admin_group;

CREATE OR REPLACE FUNCTION delete_logbuch_ds(tid integer) RETURNS void 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid master_id FROM ka_logbuch_tbl WHERE id = tid;
    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;
    DELETE FROM ka_logbuch_tbl WHERE id = tid;
END;
$$ ;

REVOKE ALL    ON FUNCTION delete_logbuch_ds(integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION delete_logbuch_ds(integer) TO GROUP :cm_group;


CREATE OR REPLACE FUNCTION delete_all_logbuch(mid integer) 
RETURNS void 
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
AS $$
BEGIN
    DELETE FROM ka_logbuch_tbl WHERE master_id = mid;
END;
$$;

REVOKE ALL     ON FUNCTION delete_all_logbuch(integer) FROM PUBLIC;
GRANT  EXECUTE ON FUNCTION delete_all_logbuch(integer) TO GROUP :admin_group;


CREATE OR REPLACE FUNCTION anonymize_logbuch(mid integer) 
RETURNS void 
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
AS $$
BEGIN
    -- DELETE FROM ka_logbuch_tbl WHERE master_id = mid;
    UPDATE ka_logbuch_tbl SET
        uuid_id    = NULL,
        bearbeiter = md5(lower(trim(both ' ' from coalesce(bearbeiter, '')))),
        kurz_notiz = NULL,
        notiz      = NULL
    WHERE master_id = mid;
END;
$$;

REVOKE ALL     ON FUNCTION anonymize_logbuch(integer) FROM PUBLIC;
GRANT  EXECUTE ON FUNCTION anonymize_logbuch(integer) TO GROUP :admin_group;


-- Wrapper Funktion die alle noetigen Funktionen zum Anonymisieren einer
-- Fallakte aufruft.
CREATE OR REPLACE FUNCTION anonymize_case_wrapper(mid integer)
RETURNS void AS
$$
BEGIN
	PERFORM anonymize_case(mid);
	PERFORM delete_all_fall_dokumente(mid);
	PERFORM delete_all_fall_termine(mid);
	PERFORM anonymize_logbuch(mid);
END;
$$ LANGUAGE plpgsql VOLATILE SECURITY DEFINER;

REVOKE ALL     ON FUNCTION anonymize_case_wrapper(integer) FROM PUBLIC;
GRANT  EXECUTE ON FUNCTION anonymize_case_wrapper(integer) TO GROUP :admin_group;

--
-- Funktionen zum Setzen der Einverstaendniserkärung 
CREATE OR REPLACE FUNCTION set_ee_status(mid integer, status integer) 
    RETURNS void 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS 
$$
BEGIN
    UPDATE master_tbl_view SET einwilligung = status WHERE id = mid;
END;
$$;

REVOKE ALL     ON FUNCTION set_ee_status(integer, integer) FROM PUBLIC;
GRANT  EXECUTE ON FUNCTION set_ee_status(integer, integer) TO GROUP :cm_group;
GRANT  EXECUTE ON FUNCTION set_ee_status(integer, integer) TO GROUP :admin_group;

-- Funktion get_default_value() gibt zu einer
-- gegebenen Relation und Spalte den Default-Wert zurueck gibt.

CREATE OR REPLACE FUNCTION get_default_value(relation_name text, attribute_name text)
RETURNS text 
LANGUAGE plpgsql STABLE AS
$$
DECLARE
    has_def bool;
    def     text;
BEGIN
    SELECT INTO has_def a.atthasdef FROM pg_class c 
    JOIN pg_attribute a ON c.oid = a.attrelid
    WHERE c.relname = relation_name AND a.attname = attribute_name;

    IF has_def IS NULL OR NOT has_def THEN
        RETURN NULL;
    END IF;

    SELECT INTO def pg_get_expr(i.adbin, i.oid) FROM 
    (SELECT c.oid, adnum, r.adbin
        FROM pg_class c 
        JOIN pg_attrdef r ON r.adrelid = c.oid
        WHERE c.relname = relation_name) i
        INNER JOIN pg_attribute p ON attrelid = i.oid AND i.adnum = p.attnum
        WHERE p.attname = attribute_name;
    RETURN def;
END;
$$;

REVOKE ALL ON FUNCTION get_default_value(text, text) FROM public;
GRANT EXECUTE ON FUNCTION get_default_value(text, text) TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION get_default_value(text, text) TO GROUP :admin_group;

CREATE OR REPLACE FUNCTION set_last_logintime(userid integer) 
    RETURNS void 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
    UPDATE ka_benutzer_tbl_view SET last_login = now() WHERE id = userid;
END;
$$ ;

REVOKE ALL ON FUNCTION set_last_logintime(integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION set_last_logintime(integer) TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION set_last_logintime(integer) TO GROUP :admin_group;

CREATE OR REPLACE FUNCTION markNewsAsRead( user_id integer, news_id integer) RETURNS VOID 
    LANGUAGE 'plpgsql' SECURITY DEFINER 
    AS $$
BEGIN
    INSERT INTO nm_benutzer_nachrichten_tbl (benutzer_id, nachrichten_id) VALUES (user_id, news_id);
END;
$$;

REVOKE ALL ON FUNCTION markNewsAsRead(user_id integer, news_id integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION markNewsAsRead(user_id integer, news_id integer) TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION markNewsAsRead(user_id integer, news_id integer) TO GROUP :admin_group;

-- Tags
CREATE OR REPLACE FUNCTION create_nm_tag_case_ds ( tagid integer, 
    masterid integer ) RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
    INSERT INTO nm_tags_cases (tag_id, master_id) 
        VALUES (tagid, masterid);
    RETURN currval('nm_tags_cases_id_seq');
END;
$$ ;

REVOKE ALL ON FUNCTION create_nm_tag_case_ds(integer, integer) 
                FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_nm_tag_case_ds(integer, integer) 
                TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION create_nm_tag_case_ds(integer, integer) 
                TO GROUP :admin_group;


CREATE OR REPLACE FUNCTION delete_nm_tag_case_ds ( tagid integer,
    masterid integer ) RETURNS VOID 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
    DELETE FROM nm_tags_cases 
            WHERE master_id = masterid AND tag_id = tagid; 
END;
$$ ;

REVOKE ALL ON FUNCTION delete_nm_tag_case_ds(integer, integer) 
                FROM PUBLIC;
GRANT EXECUTE ON FUNCTION delete_nm_tag_case_ds(integer, integer) 
                TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION delete_nm_tag_case_ds(integer, integer) 
                TO GROUP :admin_group;


CREATE OR REPLACE FUNCTION delete_tag_ds ( tagid integer) RETURNS VOID 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
    DELETE FROM tags WHERE id = tagid; 
END;
$$ ;

REVOKE ALL ON FUNCTION delete_tag_ds(integer) 
                FROM PUBLIC;
GRANT EXECUTE ON FUNCTION delete_tag_ds(integer) 
                TO GROUP :admin_group;

CREATE OR REPLACE FUNCTION create_tag_ds () RETURNS INTEGER 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
     INSERT INTO tags (label, description) 
         VALUES ('', NULL);
     RETURN currval('tags_id_seq');
END;
$$ ;

REVOKE ALL ON FUNCTION create_tag_ds() 
                FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_tag_ds() 
                TO GROUP :admin_group;

SET ROLE postgres;

CREATE OR REPLACE FUNCTION check_case(ds_id integer) RETURNS void 
AS $$
import socket

UNIX_SOCKET = "/tmp/tagd-socket"

db = plpy.execute("SELECT current_database() as db")[0]["db"]

s = None
try:
    try:
        s = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        s.connect(UNIX_SOCKET)
        s.send("%s %d\n" % (db, ds_id))
    except:
        plpy.warning('Can not connect tagd socket')
finally:
    if s:
        try: s.close()
        except: pass
          
$$ LANGUAGE plpythonu;

SET ROLE :adm_ka_owner;

COMMIT;
