BEGIN;
--
-- Tabellen fuer die Newsbox-erweiterung.
--
SET ROLE :db_adm;

INSERT INTO db_history (beschreibung, zeitpunkt)
        VALUES ('update_1_0_4/06_newsbox.sql', current_timestamp);

-- ka_nachrichten_tbl
--
-- Verwaltet Logbucheintraege zu Fallakten
--

CREATE TABLE ka_nachrichten_art_tbl (
    id         SERIAL NOT NULL,
    beschreibung VARCHAR(128),
    CONSTRAINT ka_nachrichten_art_tbl_pkey PRIMARY KEY (id)
);

INSERT INTO ka_nachrichten_art_tbl (id, beschreibung) VALUES ( -2, 'Sonstiges');
INSERT INTO ka_nachrichten_art_tbl (id, beschreibung) VALUES ( -1, 'Keine Angabe');
INSERT INTO ka_nachrichten_art_tbl (id, beschreibung) VALUES (  0, 'Systemnachricht');

CREATE TABLE ka_nachrichten_tbl (
    id         SERIAL NOT NULL,
    datum      TIMESTAMP,
    art        int4 REFERENCES ka_nachrichten_art_tbl(id) ON DELETE SET DEFAULT,
    titel      VARCHAR(128),
    nachricht  text,
    CONSTRAINT ka_nachrichten_tbl_pkey PRIMARY KEY (id)
);

--
-- Views auf die Nachrichten
--

CREATE OR REPLACE VIEW ka_nachrichten_art_tbl_view 
    AS SELECT
        a.id           AS id,
        a.beschreibung AS beschreibung
    FROM ka_nachrichten_art_tbl a;

CREATE OR REPLACE RULE ka_nachrichten_art_update_rule 
    AS ON UPDATE TO ka_nachrichten_art_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_nachrichten_art_delete_rule 
    AS ON DELETE TO ka_nachrichten_art_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_nachrichten_art_insert_rule
    AS ON INSERT TO ka_nachrichten_art_tbl_view DO INSTEAD NOTHING;

GRANT SELECT ON ka_nachrichten_art_tbl_view TO GROUP :cm_ka_group;
GRANT SELECT ON ka_nachrichten_art_tbl_view TO GROUP :admin_ka_group;

CREATE OR REPLACE VIEW ka_nachrichten_tbl_view 
    AS SELECT
        n.id           AS id,
        n.datum        AS datum,
        n.art          AS art,
        a.beschreibung AS art_beschreibung,
        n.titel        AS titel,
        n.nachricht    AS nachricht
    FROM ka_nachrichten_tbl n JOIN ka_nachrichten_art_tbl a
    ON n.art = a.id;

CREATE OR REPLACE RULE ka_nachrichten_update_rule
    AS ON UPDATE TO ka_nachrichten_tbl_view 
	DO INSTEAD
        UPDATE ka_nachrichten_tbl SET
            art        = new.art,
            titel      = new.titel,
            nachricht  = new.nachricht
	    WHERE ka_nachrichten_tbl.id = new.id;

CREATE OR REPLACE RULE ka_nachrichten_delete_rule 
    AS ON DELETE TO  ka_nachrichten_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_nachrichten_insert_rule
    AS ON INSERT TO ka_nachrichten_tbl_view DO INSTEAD NOTHING;

GRANT SELECT ON ka_nachrichten_tbl_view TO GROUP :cm_ka_group;
GRANT SELECT ON ka_nachrichten_tbl_view TO GROUP :admin_ka_group;
GRANT UPDATE ON ka_nachrichten_tbl_view TO GROUP :admin_ka_group;

ALTER TABLE ka_benutzer_tbl
    ADD COLUMN last_login timestamp;

DROP VIEW ka_benutzer_tbl_view; 
CREATE OR REPLACE VIEW ka_benutzer_tbl_view AS 
SELECT ka_benutzer_tbl.id, ka_benutzer_tbl.vorname, ka_benutzer_tbl.nachname,
ka_benutzer_tbl.telefon, ka_benutzer_tbl.aktiviert,
ka_benutzer_tbl.passwort_aendern, ka_benutzer_tbl.login,
ka_benutzer_tbl.raumnummer, ka_benutzer_tbl.filiale, p.usesysid,
ka_benutzer_tbl.vertreter, getrole(ka_benutzer_tbl.id) as rolle,
ka_benutzer_tbl.gid,
ka_benutzer_tbl.last_login
FROM ka_benutzer_tbl JOIN pg_user p
ON ka_benutzer_tbl.login = p.usename;

CREATE OR REPLACE RULE ka_benutzer_update_rule AS ON UPDATE TO ka_benutzer_tbl_view 
	DO INSTEAD  UPDATE ka_benutzer_tbl SET 
	vorname = new.vorname, 
	nachname = new.nachname, 
	telefon = new.telefon, 
	aktiviert = new.aktiviert, 
	raumnummer = new.raumnummer, 
	filiale = new.filiale, 
	passwort_aendern = new.passwort_aendern, 
	vertreter = new.vertreter,
	gid = new.gid,
	last_login = new.last_login
	WHERE ka_benutzer_tbl.id = new.id;

GRANT SELECT ON ka_benutzer_tbl_view TO GROUP :cm_ka_group;
GRANT SELECT ON ka_benutzer_tbl_view TO GROUP :admin_ka_group;
GRANT UPDATE ON ka_benutzer_tbl_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE ka_benutzer_delete_rule AS ON DELETE TO ka_benutzer_tbl_view 
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_benutzer_insert_rule AS ON INSERT TO ka_benutzer_tbl_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE FUNCTION set_last_logintime(userid integer) 
    RETURNS void 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
    UPDATE ka_benutzer_tbl_view SET last_login = now() WHERE id = userid;
END;
$$ ;

REVOKE ALL ON FUNCTION set_last_logintime(integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION set_last_logintime(integer) TO GROUP :cm_ka_group;
GRANT EXECUTE ON FUNCTION set_last_logintime(integer) TO GROUP :admin_ka_group;

COMMIT;
