BEGIN;

--
-- Angleichung der Angaben des Status des junegen Menschen bei ein und
-- Austritt 
-- Issue919
--

ALTER TABLE master_tbl DISABLE TRIGGER USER;

SET ROLE :db_adm;

INSERT INTO db_history (beschreibung, zeitpunkt)
       VALUES ('2.0/02_eintritt-austritt.sql', current_timestamp);

-- Konvertierung Austritte
-- [Ax] = vermittlung in master_tbl (kommt aus dem alten Datenmodell)
-- [Sx] = ein_eintritt_ist_der_jugendliche bzw. bei_austritt_ist_der_jugendliche
-- [B01] = schulart
-- [B02] = ausbildung 
-- [B03] = umschulung 

-- 10. [b01a-d] -> [S01] + [S22]
-- [S01] = 15 | allgemeinbildende Schule 
-- [b01a-d] =  
-- id |           value           
------+---------------------------
-- -1 | Keine Angabe
--  0 | Sonderschule/Förderschule
--  1 | Hauptschule
--  2 | Gesamtschule 
--  3 | Realschule
--  4 | Gymnasium
--  5 | Berufsfachschule
--  6 | sonstige Schule 

UPDATE master_tbl set bei_austritt_ist_der_jugendliche = 15 
WHERE ( schulart in (0,1,2,3,4));
UPDATE master_tbl set bei_austritt_ist_der_jugendliche_konkretisierung = 'Sonderschule/Förderschule' where schulart = 0;
UPDATE master_tbl set bei_austritt_ist_der_jugendliche_konkretisierung = 'Hauptschule' where schulart = 1;
UPDATE master_tbl set bei_austritt_ist_der_jugendliche_konkretisierung = 'Gesamtschule' where schulart = 2;
UPDATE master_tbl set bei_austritt_ist_der_jugendliche_konkretisierung = 'Realschule' where schulart = 3;
UPDATE master_tbl set bei_austritt_ist_der_jugendliche_konkretisierung = 'Gymnasium' where schulart = 4;

-- 11. [b01a-d] -> [S02] + [S22]
-- [S02] = 7 | andere Schule (z.B. Berufsfachschule 
-- [b01a-d] =  
UPDATE master_tbl set bei_austritt_ist_der_jugendliche = 7 
WHERE ( schulart in (5,6));
UPDATE master_tbl set bei_austritt_ist_der_jugendliche_konkretisierung = 'Berufsfachschule' where schulart = 5;
UPDATE master_tbl set bei_austritt_ist_der_jugendliche_konkretisierung = schule_sonstiges_nennung where schulart = 6;

-- 12. [b02a-b] -> [S03]
-- [S03] = 16 | duale Berufsausbildung (auch Verbundausbildung) 
-- [b02a-b] =  0 | betrieblich
--             1 | Verbundausbildung 
UPDATE master_tbl set bei_austritt_ist_der_jugendliche = 16 
WHERE ( ausbildung in (0,1));

-- 13. [b02c] -> [S05]
-- [S05] = 9 | geförderte Berufsausbildung (BaE) 
-- [b02c] = 2 | außerbetrieblich 
UPDATE master_tbl set bei_austritt_ist_der_jugendliche = 9 
WHERE ( ausbildung in (2));

-- 14. [b02d] -> [S04]
-- [S04] = 8 | schulische Berufsausbildung 3 | schulisch
-- [b02d] =  
UPDATE master_tbl set bei_austritt_ist_der_jugendliche = 8 
WHERE ( ausbildung in (3));

-- 15. [b02e] -> [S06] + [S22]
-- [S06] = 1 | in Ausbildung ohne genauere Angabe 
-- [S22] = bei_austritt_ist_der_jugendliche_konkretisierung 
-- [b02d] = -2 | Sonstiges 
UPDATE master_tbl set bei_austritt_ist_der_jugendliche = 1,
bei_austritt_ist_der_jugendliche_konkretisierung = ausbildung_sonstiges_nennung 
WHERE ( ausbildung in (-2));

-- 16. [b05a-b] -> [S10] + [S22]
-- [S10] = 2 | in Arbeit ohne Angabe zum Beschäftigungsumfang 
-- [S22] = bei_austritt_ist_der_jugendliche_konkretisierung 
-- [b05a-d] = 0 | festangestellt unbefristet
--	      1 | festangestellt befristet 
UPDATE master_tbl set bei_austritt_ist_der_jugendliche = 2 WHERE ( erwerbstatus in (0,1));
UPDATE master_tbl set bei_austritt_ist_der_jugendliche_konkretisierung = 'festangestellt unbefristet' WHERE ( erwerbstatus in (0));
UPDATE master_tbl set bei_austritt_ist_der_jugendliche_konkretisierung = 'festangestellt befristet' WHERE ( erwerbstatus in (1));

-- 17. [b05c-d] -> [S11]
-- [S11] = 6 | Selbstständig / freiberuflich 
-- [b05c-d] = 2 | freiberuflich
--	      3 | selbstständig 
UPDATE master_tbl set bei_austritt_ist_der_jugendliche = 6 WHERE ( erwerbstatus in (2,3));

-- 18. [b05e] -> [S11]
-- [S11] = 6 | Selbstständig / freiberuflich 
-- [b05c-d] = 2 | freiberuflich
--	      3 | selbstständig 
UPDATE master_tbl set bei_austritt_ist_der_jugendliche = 6 WHERE ( erwerbstatus in (2,3));

-- 19. [b05e] -> [S20] + [S22]
-- [S20] = -2 | Sonstiges 
-- [b05e] = -2 | Sonstiges 
UPDATE master_tbl set bei_austritt_ist_der_jugendliche = -2,
bei_austritt_ist_der_jugendliche_konkretisierung = erwerbsstatus_sonstiges_nennung 
WHERE ( erwerbstatus in (-2));

-- 20. [b06a] -> [S07]
-- [S07] = 12 | Vollzeit beschäftigt 
-- [b06a] = 1 | vollzeitbeschäftigt 
UPDATE master_tbl set bei_austritt_ist_der_jugendliche = 12
WHERE ( beschaeftigungsumfang in (1));

-- 21. [b06b] -> [S08]
-- [S08] = 13 | Teilzeit beschäftigt 
-- [b06b] = 0 | teilzeitbeschäftigt 
UPDATE master_tbl set bei_austritt_ist_der_jugendliche = 13
WHERE ( beschaeftigungsumfang in (0));

-- 22. [b06c] -> [S09]
-- [S09] = 14 | Geringfügig beschäftigt
-- [b06b] = 2 | geringfügig beschäftig 
UPDATE master_tbl set bei_austritt_ist_der_jugendliche = 14
WHERE ( beschaeftigungsumfang in (2));

-- 23. [b07a] -> [S14]
-- [S14] = 3 | arbeitslos (gemeldet) 
-- [b07a] = 0 | Arbeitslosigkeit 
UPDATE master_tbl set bei_austritt_ist_der_jugendliche = 3
WHERE ( kategorien_nichterwerbsarbeit_nichtausbildung in (0));

-- 24. [b07b] -> [S15]
-- [S15] = 4 | Erziehungszeit 
-- [b07b] = 1 | Hausarbeit/Familienarbeit/Kindererziehung
UPDATE master_tbl set bei_austritt_ist_der_jugendliche = 4
WHERE ( kategorien_nichterwerbsarbeit_nichtausbildung in (1));

-- 25. [b07c-h] -> [S16]
-- [S16] = 11 | kein Status (z.B. Ausbildungsplatz suchend, nicht arbeitslos gemeldet) 
-- [b07c-h] = 2 | Wehr-/Zivildienst
--            3 | Freiwilliges Soziales Jahr und ähnliches
--            4 | Strafvollzug
--            5 | Verlassen der BRD
--            6 | noch offen
--            -2 | Sonstiges
UPDATE master_tbl set bei_austritt_ist_der_jugendliche = 11
WHERE ( kategorien_nichterwerbsarbeit_nichtausbildung in (2,3,4,5,6,-2));
UPDATE master_tbl set bei_austritt_ist_der_jugendliche_konkretisierung = 'Wehr-/Zivildienst' WHERE (kategorien_nichterwerbsarbeit_nichtausbildung in (2));
UPDATE master_tbl set bei_austritt_ist_der_jugendliche_konkretisierung = 'Freiwilliges Soziales Jahr und ähnliches' WHERE (kategorien_nichterwerbsarbeit_nichtausbildung in (3));
UPDATE master_tbl set bei_austritt_ist_der_jugendliche_konkretisierung = 'Strafvollzug' WHERE (kategorien_nichterwerbsarbeit_nichtausbildung in (4));
UPDATE master_tbl set bei_austritt_ist_der_jugendliche_konkretisierung = 'Verlassen der BRD' WHERE (kategorien_nichterwerbsarbeit_nichtausbildung in (5));
UPDATE master_tbl set bei_austritt_ist_der_jugendliche_konkretisierung = 'noch offen' WHERE (kategorien_nichterwerbsarbeit_nichtausbildung in (6));
UPDATE master_tbl set bei_austritt_ist_der_jugendliche_konkretisierung = kategorien_sonstiges_nennung WHERE (kategorien_nichterwerbsarbeit_nichtausbildung in (-2));

-- -- In jedem Fall sollen die Angaben bei der Vermittlung maßgebend sein.
-- -- Daher wird diese Konvertierung zum Schluss durchgeführt um ggf.
-- -- Wiedersprüchliche Angaben zu überschreiben 

-- 1. [A01] oder [b01a-d] -> [S01]
-- [S01] = allgemeinbildende Schule (15)
-- [A01] = in die allgemein bildende Schule (0)
-- [b01a-d] =  
-- id |           value           
------+---------------------------
-- -1 | Keine Angabe
--  0 | Sonderschule/Förderschule
--  1 | Hauptschule
--  2 | Gesamtschule 
--  3 | Realschule
--  4 | Gymnasium
--  5 | Berufsfachschule
--  6 | sonstige Schule 
UPDATE master_tbl set bei_austritt_ist_der_jugendliche = 15 
WHERE (vermittlung = 0) OR ( schulart in (0,1,2,3,4)); 


-- 2. [A02] -> [S02]
-- [S02] = 7 | andere Schule (z.B. Berufsfachschule) 
-- [A02] = 7 | in die Berufsfachschule 
UPDATE master_tbl set bei_austritt_ist_der_jugendliche = 7 
WHERE (vermittlung = 7); 

-- 3. [A03] -> [S13]
-- [S13] = 5 | sonstige Maßnahme (z.B. EQJ) 
-- [A03] = 1 | in eine Maßnahme 
UPDATE master_tbl set bei_austritt_ist_der_jugendliche = 5 
WHERE (vermittlung = 1);

-- 4. [A04] -> [S12]
-- [S13] = 10 | berufsvorbereitende Bildungsnaßnahmen: z.B. BvB (Bund), BVJ/BGJ (Länder) 
-- [A04] = 5 | in das Berufsvorbereitungsjahr (BVJ) 
UPDATE master_tbl set bei_austritt_ist_der_jugendliche = 10 
WHERE (vermittlung = 5); 

-- 5. [A05] -> [S12]
-- [S13] = 10 | berufsvorbereitende Bildungsnaßnahmen: z.B. BvB (Bund), BVJ/BGJ (Länder) 
-- [A05] = 6 | in das Berufsgrundbildungsjahr (BGJ 
UPDATE master_tbl set bei_austritt_ist_der_jugendliche = 10 
WHERE (vermittlung = 6); 

-- 6. [A06] -> [S06]
-- [S06] = 1 | in Ausbildung ohne genauere Angabe 
-- [A06] = 2 | in die Ausbildung 
UPDATE master_tbl set bei_austritt_ist_der_jugendliche = 1 
WHERE (vermittlung = 2); 

-- 7. [A07] -> [S10]
-- [S10] = 2 | in Arbeit ohne Angabe zum Beschäftigungsumfang
-- [A07] = 3 | in die Arbeit 
UPDATE master_tbl set bei_austritt_ist_der_jugendliche = 2 
WHERE (vermittlung = 3); 

-- 8. [A08] -> [S20] + [S21]
-- [A08] = -2 | Sonstiges
-- [S20] = -2 | sonstiges
-- [S21] = bei_austritt_ist_der_jugendliche_nennung
UPDATE master_tbl set bei_austritt_ist_der_jugendliche = -2, bei_austritt_ist_der_jugendliche_nennung = vermittlung_nennung 
WHERE (vermittlung = -2); 

-- 9. [A09] -> [S20] + [S22]
-- [A09] = 4 | gar nicht vermittel 
-- [S20] = -2 | sonstiges
-- [S22] = bei_austritt_ist_der_jugendliche_konkretisierung 
UPDATE master_tbl set bei_austritt_ist_der_jugendliche = -2,
bei_austritt_ist_der_jugendliche_konkretisierung = 'gar nicht vermittelt' 
WHERE (vermittlung = 4); 

-- Das Feld "klassenstufe_1" entfällt im neuen Datenmodell. 
-- Die Information soll in das Feld "Weitere Angaben zum Status" übernommen
-- werden sofern der Übergang in die Schule ist.
UPDATE master_tbl 
set bei_austritt_ist_der_jugendliche_konkretisierung =
bei_austritt_ist_der_jugendliche_konkretisierung || '; Klassenstufe: ' ||
coalesce(klassenstufe_1::varchar, 'unbekannt')
WHERE bei_austritt_ist_der_jugendliche IN (15,7);


SET ROLE postgres;
ALTER TABLE master_tbl ENABLE TRIGGER USER;

COMMIT;
