BEGIN;

--
-- Konvertierung der Angaben in der Abschlussbewertung 
-- Issue904
--

ALTER TABLE master_tbl DISABLE TRIGGER USER;

SET ROLE :db_adm;

INSERT INTO db_history (beschreibung, zeitpunkt)
       VALUES ('2.0/03_beendigung.sql', current_timestamp);

-- ALT
-- 
-- gruende_abbruch
-- id |                value                 
------+--------------------------------------
-- -2 | Sonstiges
-- -1 | Keine Angabe
--  0 | persönliche Gründe
--  1 | Umzug
--  2 | Krankheit
--  3 | Altersüberschreitung
--  4 | mangelnde Mitarbeit des Jugendlichen
--  5 | Konflikte
--  6 | Haft/U-Haft
--
-- abbruch_durch
-- id |            value            
------+-----------------------------
-- -1 | Keine Angabe
--  0 | Jugendliche
--  1 | Kompetenzagentur
--  2 | Sorgeberechtigte
--  3 | beidseitiges Einverständnis

-- NEU
--
-- gruende_vorz_beendigung
-- id |        value         
------+----------------------
-- -3 | Unbekannt
-- -2 | Sonstiges
-- -1 | Keine Angabe
--  0 | Schwangerschaft
--  1 | Umzug
--  2 | Krankheit
--  3 | Altersüberschreitung
--  4 | Haft/U-Haf
--
-- gruende_abbruch
-- id |                value                 
------+--------------------------------------
-- -3 | Unbekannt
-- -2 | Sonstiges
-- -1 | Keine Angabe
--  0 | Kontaktabbruch
--  1 | mangelnde Mitarbeit des Jugendlichen
--  2 | Konflikte
--
-- vorz_beendigung_durch
-- id |            value            
------+-----------------------------
-- -3 | Unbekannt
-- -2 | Sonstiges
-- -1 | Keine Angabe
--  0 | Jugendliche
--  2 | Sorgeberechtigte
--  3 | beidseitiges Einverständnis
--
-- abbruch_durch
-- id |            value            
------+-----------------------------
-- -3 | Unbekannt
-- -2 | Sonstiges
-- -1 | Keine Angabe
--  0 | Jugendliche
--  1 | Kompetenzagentur
--  3 | beidseitiges Einverständnis

-- Behandlung ungültiger Kombinationen
--
UPDATE master_tbl set abbruch_durch = -1 where art_beendigung = 0;
UPDATE master_tbl set gruende_abbruch = -1 where art_beendigung = 0;
UPDATE master_tbl set gruende_sonstiges_nennung_4 = NULL where art_beendigung = 0;

-- Umwandlung einer vorzeitigen Beendigung zu einem Abbruch bei Angabe
-- bestimmter Gründe:
-- * Kontaktabbruch <- fehlt
-- * mangelnde Mitarbeit des Jugendlichen
-- * Konflikte <- neu hinzugenommen
-- * Sonstiges
UPDATE master_tbl set art_beendigung = 1 WHERE art_beendigung = 2 
AND (gruende_abbruch in (4, -2, 5));

-- Wurden als GRÜNDE angegeben:
-- * Schwangerschaft <- fehlt
-- * Umzug
-- * Krankheit
-- * Altersüberschreitung
-- * Haft/U-Haft
-- in Kombination mit ABBRUCH, so bleibt die Art der Beendigung erhalten.
-- Allerdings werden die Gründe geändert in "Sonstige Gründe" und der
-- entsprechende Eintrag ins Textfeld geschrieben.
UPDATE master_tbl set gruende_vorz_abbruch_sonstiges_nennung = 'Umzug' WHERE
art_beendigung = 1
AND gruende_abbruch = 1;
UPDATE master_tbl set gruende_vorz_abbruch_sonstiges_nennung = 'Krankheit' WHERE
art_beendigung = 1
AND gruende_abbruch = 2;
UPDATE master_tbl set gruende_vorz_abbruch_sonstiges_nennung = 'Altersüberschreitung' WHERE
art_beendigung = 1
AND gruende_abbruch = 3;
UPDATE master_tbl set gruende_vorz_abbruch_sonstiges_nennung = 'Haft/U-Haft' WHERE
art_beendigung = 1
AND gruende_abbruch = 6;
UPDATE master_tbl set gruende_abbruch = -2 WHERE art_beendigung = 1 
AND (gruende_abbruch in (1,2,3,6));

-- Bei VORZEITIGE BEENDIGUNG
-- + Beendigung durch: KA  --> Beendigung durch [Sonstige] + Textfeld "Beendigung
-- durch Sonstige" [Kompetenzagentur]
UPDATE master_tbl set vorz_beendigung_durch = -2, vorz_beendigung_sonstiges_nennung = 'Kompetenzagentur' 
WHERE art_beendigung = 2 and abbruch_durch = 1;

-- Bei ABBRUCH 
-- + Beendigung durch: Sorgeberechtigte --> Beendigung durch [Sonstige] + Textfeld
-- "Beendigung durch Sonstige" [Sorgeberechtigte]
UPDATE master_tbl set abbruch_durch = -2, abbruch_sonstiges_nennung = 'Sorgeberechtigte' 
WHERE art_beendigung = 1 and abbruch_durch = 2;

-- Übernahme valider Kombinationen 
--
-- Abbrüche
-- Gruende für den Abbruch
UPDATE master_tbl set gruende_abbruch = -2,
gruende_vorz_abbruch_sonstiges_nennung = array_to_string(ARRAY[gruende_sonstiges_nennung_4, gruende_vorz_abbruch_sonstiges_nennung], ';')
WHERE gruende_abbruch = -2 and art_beendigung = 1;

UPDATE master_tbl 
set gruende_abbruch = -2, 
gruende_vorz_abbruch_sonstiges_nennung = array_to_string(ARRAY['persönliche Gründe', gruende_vorz_abbruch_sonstiges_nennung], ';')
WHERE gruende_abbruch = 0 and (art_beendigung = 1 OR art_beendigung = -1); 

UPDATE master_tbl set gruende_abbruch = 1 WHERE gruende_abbruch = 4 and
(art_beendigung = 1 OR art_beendigung = -1); 
UPDATE master_tbl set gruende_abbruch = 2 WHERE gruende_abbruch = 5 and
(art_beendigung = 1 OR art_beendigung = -1); 
-- Abbruch durch
UPDATE master_tbl set abbruch_durch = 0 WHERE abbruch_durch = 0 and art_beendigung = 1;
UPDATE master_tbl set abbruch_durch = 1 WHERE abbruch_durch = 1 and art_beendigung = 1;
UPDATE master_tbl set abbruch_durch = 3 WHERE abbruch_durch = 3 and art_beendigung = 1; 

-- Vorzeitige Beendigung 
-- Gruende für vorz. Beendigung
UPDATE master_tbl set gruende_vorz_beendigung = -2, gruende_abbruch = -1 WHERE gruende_abbruch = -2 and art_beendigung = 2;

UPDATE master_tbl 
set gruende_vorz_beendigung = -2, 
gruende_vorz_beendigung_sonstiges_nennung = array_to_string(ARRAY['persönliche Gründe', gruende_vorz_abbruch_sonstiges_nennung], ';')
WHERE gruende_abbruch = 0 and art_beendigung = 2; 

UPDATE master_tbl set gruende_vorz_beendigung = 1, gruende_abbruch = -1 
  WHERE gruende_abbruch = 1 and (art_beendigung = 2 OR art_beendigung = -1); 
UPDATE master_tbl set gruende_vorz_beendigung = 2, gruende_abbruch = -1 
  WHERE gruende_abbruch = 2 and (art_beendigung = 2 OR art_beendigung = -1);
UPDATE master_tbl set gruende_vorz_beendigung = 3, gruende_abbruch = -1 
  WHERE gruende_abbruch = 3 and (art_beendigung = 2 OR art_beendigung = -1);
UPDATE master_tbl set gruende_vorz_beendigung = 4, gruende_abbruch = -1 
  WHERE gruende_abbruch = 6 and (art_beendigung = 2 OR art_beendigung = -1);
-- Abbruch durch
UPDATE master_tbl set vorz_beendigung_durch = 0, abbruch_durch = -1 WHERE abbruch_durch = 0 and art_beendigung = 2;
--UPDATE master_tbl set vorz_beendigung_durch = 1, abbruch_durch = -1 WHERE abbruch_durch = 1 and art_beendigung = 2; 
UPDATE master_tbl set vorz_beendigung_durch = 2, abbruch_durch = -1 WHERE abbruch_durch = 2 and art_beendigung = 2;
UPDATE master_tbl set vorz_beendigung_durch = 3, abbruch_durch = -1 WHERE abbruch_durch = 3 and art_beendigung = 2;

-- Beendigung (durch), wenn keine genaue Art der Beendigung angegeben:
UPDATE master_tbl 
  SET vorz_beendigung_durch = 0, abbruch_durch = -1
  WHERE abbruch_durch = 0 and art_beendigung = -1
    AND gruende_vorz_beendigung in (-2,0,1,2,3,4);
UPDATE master_tbl 
  SET vorz_beendigung_durch = 2, abbruch_durch = -1
  WHERE abbruch_durch = 2 and art_beendigung = -1
    AND gruende_vorz_beendigung in (-2,0,1,2,3,4);
UPDATE master_tbl 
  SET vorz_beendigung_durch = 3, abbruch_durch = -1
  WHERE abbruch_durch = 3 and art_beendigung = -1
    AND gruende_vorz_beendigung in (-2,0,1,2,3,4);

-- Letzte Behandlung von abbruch_durch = 2 (Sorgeberechtigte)
UPDATE master_tbl 
  SET vorz_beendigung_durch = 2, abbruch_durch = -1
  WHERE abbruch_durch = 2 and art_beendigung = -1;


SET ROLE postgres;
ALTER TABLE master_tbl ENABLE TRIGGER USER;

COMMIT;
