BEGIN;

--
-- Umwandung der Alteinträge aus dem Bereich "gesundheitliche" Situation
-- in die RG Benachteiligungen. 
-- Issue886
--

ALTER TABLE master_tbl DISABLE TRIGGER USER;

SET ROLE :db_adm;

INSERT INTO db_history (beschreibung, zeitpunkt)
       VALUES ('2.0/04_soziale_benachteiligung.sql', current_timestamp);

-- ALT
-- arzt_gutachten_physisch
-- physische_beeintraechtigungen
-- physische_beeintraechtigungen_text

-- arzt_gutachten_psychisch
-- psychische_beeintraechtigungen
-- psychische_beeintraechtigungen_text

-- anerkannte_behinderung

-- NEU
-- rg_benachteiligung_tbl

CREATE OR REPLACE FUNCTION create_rg_benachteiligung_tbl(pid integer, nuuid char(36))
RETURNS integer
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$
DECLARE
    rid integer;
    mid integer;
BEGIN
    mid := pid;

    IF NOT isEditorOrStandin(mid) THEN
        -- RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    INSERT INTO rg_benachteiligung_tbl (master_id) VALUES (pid);

    SELECT INTO rid currval('rg_benachteiligung_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE rg_benachteiligung_tbl
        SET    uuid_id = nuuid
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('rg_benachteiligung_tbl', rid);
        
    END IF;

    RETURN rid;
END;
$$;

-- Übernahme der alten Einträge in eine RG
-- Für alle Eintrage in der master_tbl
CREATE OR REPLACE FUNCTION insert_rg() RETURNS VOID AS $$
DECLARE
  r RECORD;
  rgid INTEGER;
BEGIN
  FOR r in SELECT id, arzt_gutachten_physisch, physische_beeintraechtigungen,
  physische_beeintraechtigungen_text, arzt_gutachten_psychisch,
  psychische_beeintraechtigungen, psychische_beeintraechtigungen_text,
  anerkannte_behinderung
  FROM master_tbl LOOP
    -- 1. Übernahme physische Beeinträchtigung
    IF r.physische_beeintraechtigungen = 1 THEN
      -- Insert new RG
      rgid = create_rg_benachteiligung_tbl(r.id, NULL);
      ---- Update values in RG
      UPDATE rg_benachteiligung_tbl set 
      rg_benachteiligung_bereich = 6,
      rg_benachteiligung_merkmal_g = -3  
      where id = rgid;
      RAISE NOTICE 'rgid is %', rgid;
    END IF;
    -- 2. Übernahme Sonstige physische Beeinträchtigung
    IF r.physische_beeintraechtigungen_text <> '' THEN
      -- Insert new RG
      rgid = create_rg_benachteiligung_tbl(r.id, NULL);
      ---- Update values in RG
      UPDATE rg_benachteiligung_tbl set 
      rg_benachteiligung_bereich = 6,
      rg_benachteiligung_merkmal_g = -2,  
      rg_benachteiligung_merkmal_g_text = r.physische_beeintraechtigungen_text  
      where id = rgid;
      RAISE NOTICE 'rgid is %', rgid;
    END IF;
    -- 3. Übernahme psychische Beeinträchtigung
    IF r.psychische_beeintraechtigungen = 1 THEN
      -- Insert new RG
      rgid = create_rg_benachteiligung_tbl(r.id, NULL);
      ---- Update values in RG
      UPDATE rg_benachteiligung_tbl set 
      rg_benachteiligung_bereich = 7,
      rg_benachteiligung_merkmal_h = -3  
      where id = rgid;
      RAISE NOTICE 'rgid is %', rgid;
    END IF;
    -- 4. Übernahme Sonstige psychische Beeinträchtigung
    IF r.psychische_beeintraechtigungen_text <> '' THEN
      -- Insert new RG
      rgid = create_rg_benachteiligung_tbl(r.id, NULL);
      ---- Update values in RG
      UPDATE rg_benachteiligung_tbl set 
      rg_benachteiligung_bereich = 7,
      rg_benachteiligung_merkmal_h = -2,
      rg_benachteiligung_merkmal_h_text = r.psychische_beeintraechtigungen_text 
      where id = rgid;
      RAISE NOTICE 'rgid is %', rgid;
    END IF;
    -- 5. Übernahme anerkannte Behinderung
    IF r.anerkannte_behinderung = 1 THEN
      -- Insert new RG
      rgid = create_rg_benachteiligung_tbl(r.id, NULL);
      ---- Update values in RG
      UPDATE rg_benachteiligung_tbl set 
      rg_benachteiligung_bereich = 6,
      rg_benachteiligung_merkmal_g = 0,
      rg_arzt_gutachten_physisch = r.arzt_gutachten_physisch,
      rg_anerkannte_behinderung = r.anerkannte_behinderung
      where id = rgid;
      RAISE NOTICE 'rgid is %', rgid;
    END IF;
    -- 6. In jedem Fall wird eine leere RG  erstellt. 
    rgid = create_rg_benachteiligung_tbl(r.id, NULL);
  END LOOP;
END;
$$ LANGUAGE plpgsql;
SELECT insert_rg();
DROP FUNCTION insert_rg();

CREATE OR REPLACE FUNCTION create_rg_benachteiligung_tbl(pid integer, nuuid char(36))
RETURNS integer
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$
DECLARE
    rid integer;
    mid integer;
BEGIN
    mid := pid;

    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    INSERT INTO rg_benachteiligung_tbl (master_id) VALUES (pid);

    SELECT INTO rid currval('rg_benachteiligung_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE rg_benachteiligung_tbl
        SET    uuid_id = nuuid
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('rg_benachteiligung_tbl', rid);
        
    END IF;

    RETURN rid;
END;
$$;

SET ROLE postgres;
ALTER TABLE master_tbl ENABLE TRIGGER USER;

COMMIT;
