BEGIN;

--
-- Umwandlung der Tabellen für die Unterstützungsanangebote in RG 
-- Issue893
--

ALTER TABLE master_tbl DISABLE TRIGGER USER;

-- GRANT READ READ to PUBLIC for rg_tables
GRANT SELECT ON TABLE rg_angebote_berufliche_qualifizierung_tbl TO PUBLIC; 
GRANT SELECT ON TABLE rg_angebote_berufsvorbereitung_tbl TO PUBLIC; 
GRANT SELECT ON TABLE rg_angebote_lebensbewaeltigung_tbl TO PUBLIC; 
GRANT SELECT ON TABLE rg_angebote_bildenden_bereich_tbl TO PUBLIC; 

SET ROLE :db_adm;

INSERT INTO db_history (beschreibung, zeitpunkt)
       VALUES ('2.0/01_rgunterstuetzung.sql', current_timestamp);

-- Create modified create_rg_* functions without DSID checks.
CREATE OR REPLACE FUNCTION create_rg_unterstuetzungsangebote_tbl(pid integer, nuuid char(36))
RETURNS integer
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$
DECLARE
    rid integer;
    mid integer;
BEGIN
    mid := pid;

    IF NOT isEditorOrStandin(mid) THEN
        -- RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    INSERT INTO rg_unterstuetzungsangebote_tbl (master_id) VALUES (pid);

    SELECT INTO rid currval('rg_unterstuetzungsangebote_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE rg_unterstuetzungsangebote_tbl
        SET    uuid_id = nuuid
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('rg_unterstuetzungsangebote_tbl', rid);
        
    END IF;

    RETURN rid;
END;
$$;


-- Übernahme der alten Einträge in eine RG
-- Für alle Eintrage in der master_tbl
CREATE OR REPLACE FUNCTION insert_rg() RETURNS VOID AS $$
DECLARE
  r RECORD;
  rgid INTEGER;
BEGIN
  -- rg_angebote_berufliche_qualifizierung_tbl
  FOR r in SELECT * 
  FROM rg_angebote_berufliche_qualifizierung_tbl LOOP
      rgid = create_rg_unterstuetzungsangebote_tbl(r.master_id, r.uuid_id);
      UPDATE rg_unterstuetzungsangebote_tbl set
      art_angebot = 3,
      angebotsform = r.angebotsform_2,
      -- angebotsform_nennung
      -- art_der_unterstuetzung
      -- art_der_unterstuetzung_1
      -- unterstuetzung_sonstiges_nennung_1
      angebote_im_bereich = r.angebote_im_bereich,
      angebot_sonstiges_nennung = r.unterstuetzung_sonstiges_nennung_1,
      -- art_der_unterstuetzung_2
      -- unterstuetzung_sonstiges_nennung_2
      durchfuehrende_institution = r.durchfuehrende_institution_2,

      zielsetzungangebot = r.zielsetzungangebot,
      indikator_1 = r.indikator_1,
      bewertung_manager_1 = r.bewertung_manager_1,
      bewertung_jugendlicher_1 = r.bewertung_jugendlicher_1,
      indikator_2 = r.indikator_2,
      bewertung_manager_2 = r.bewertung_manager_2,
      bewertung_jugendlicher_2 = r.bewertung_jugendlicher_2,
      indikator_3 = r.indikator_3,
      bewertung_manager_3 = r.bewertung_manager_3,
      bewertung_jugendlicher_3 = r.bewertung_jugendlicher_3,
      zielerreichung = r.zielerreichung,

      unterstuetzung_start = r.unterstuetzung_start_2,
      unterstuetzung_ende = r.unterstuetzung_ende_2,
      zeitlicher_umfang_unterstuetzung = r.zeitlicher_umfang_unterstuetzung_2,
      beendigung_1 = r.beendigung_3,
      beendigung_durch = r.beendigung_durch_2,
      gruende_vorzeitige_beendigung = r.gruende_vorzeitige_beendigung_2,
      gruende_sonstiges_nennung = r.gruende_sonstiges_nennung_2,
      nachweis_bescheinigung = r.nachweis_bescheinigung_2,
      sonstiges_nachweis_bescheinigung = r.sonstiges_nachweis_bescheinigung_2
      WHERE id = rgid;
  END LOOP;
  -- rg_angebote_berufsvorbereitung_tbl
  FOR r in SELECT * 
  FROM rg_angebote_berufsvorbereitung_tbl LOOP
      rgid = create_rg_unterstuetzungsangebote_tbl(r.master_id, r.uuid_id);
      UPDATE rg_unterstuetzungsangebote_tbl set
      art_angebot = 2,
      angebotsform = r.angebotsform_1,
      -- angebotsform_nennung
      -- art_der_unterstuetzung
      art_der_unterstuetzung_1 = r.art_der_unterstuetzung_1,
      unterstuetzung_sonstiges_nennung_1 = r.unterstuetzung_sonstiges_nennung,
      -- angebote_im_bereich
      -- angebot_sonstiges_nennung
      -- art_der_unterstuetzung_2
      -- unterstuetzung_sonstiges_nennung_2
      durchfuehrende_institution = r.durchfuehrende_institution_1,

      zielsetzungangebot = r.zielsetzungangebot,
      indikator_1 = r.indikator_1,
      bewertung_manager_1 = r.bewertung_manager_1,
      bewertung_jugendlicher_1 = r.bewertung_jugendlicher_1,
      indikator_2 = r.indikator_2,
      bewertung_manager_2 = r.bewertung_manager_2,
      bewertung_jugendlicher_2 = r.bewertung_jugendlicher_2,
      indikator_3 = r.indikator_3,
      bewertung_manager_3 = r.bewertung_manager_3,
      bewertung_jugendlicher_3 = r.bewertung_jugendlicher_3,
      zielerreichung = r.zielerreichung,

      unterstuetzung_start = r.unterstuetzung_start_1,
      unterstuetzung_ende = r.unterstuetzung_ende_1,
      zeitlicher_umfang_unterstuetzung = r.zeitlicher_umfang_unterstuetzung_1,
      beendigung_1 = r.beendigung_2,
      beendigung_durch = r.beendigung_durch_1,
      gruende_vorzeitige_beendigung = r.gruende_vorzeitige_beendigung_1,
      gruende_sonstiges_nennung = r.gruende_sonstiges_nennung_1,
      nachweis_bescheinigung = r.nachweis_bescheinigung_1,
      sonstiges_nachweis_bescheinigung = r.sonstiges_nachweis_bescheinigung_1
      WHERE id = rgid;
  END LOOP;
  -- rg_angebote_bildenden_bereich_tbl
  FOR r in SELECT * 
  FROM rg_angebote_bildenden_bereich_tbl LOOP
      rgid = create_rg_unterstuetzungsangebote_tbl(r.master_id, r.uuid_id);
      UPDATE rg_unterstuetzungsangebote_tbl set
      art_angebot = 1,
      angebotsform = r.angebotsform,
      -- angebotsform_nennung
      art_der_unterstuetzung = r.art_der_unterstuetzung,
      -- art_der_unterstuetzung_1
      -- unterstuetzung_sonstiges_nennung_1
      -- angebote_im_bereich
      -- angebot_sonstiges_nennung
      -- art_der_unterstuetzung_2
      -- unterstuetzung_sonstiges_nennung_2
      durchfuehrende_institution = r.durchfuehrende_institution,

      zielsetzungangebot = r.zielsetzungangebot,
      indikator_1 = r.indikator_1,
      bewertung_manager_1 = r.bewertung_manager_1,
      bewertung_jugendlicher_1 = r.bewertung_jugendlicher_1,
      indikator_2 = r.indikator_2,
      bewertung_manager_2 = r.bewertung_manager_2,
      bewertung_jugendlicher_2 = r.bewertung_jugendlicher_2,
      indikator_3 = r.indikator_3,
      bewertung_manager_3 = r.bewertung_manager_3,
      bewertung_jugendlicher_3 = r.bewertung_jugendlicher_3,
      zielerreichung = r.zielerreichung,

      unterstuetzung_start = r.unterstuetzung_start,
      unterstuetzung_ende = r.unterstuetzung_ende,
      zeitlicher_umfang_unterstuetzung = r.zeitlicher_umfang_unterstuetzung,
      beendigung_1 = r.beendigung_1,
      beendigung_durch = r.beendigung_durch,
      gruende_vorzeitige_beendigung = r.gruende_vorzeitige_beendigung,
      gruende_sonstiges_nennung = r.gruende_sonstiges_nennung,
      nachweis_bescheinigung = r.nachweis_bescheinigung,
      sonstiges_nachweis_bescheinigung = r.sonstiges_nachweis_bescheinigung
      WHERE id = rgid;
  END LOOP;
  -- rg_angebote_lebensbewaeltigung_tbl
  FOR r in SELECT * 
  FROM rg_angebote_lebensbewaeltigung_tbl LOOP
      rgid = create_rg_unterstuetzungsangebote_tbl(r.master_id, r.uuid_id);
      UPDATE rg_unterstuetzungsangebote_tbl set
      art_angebot = 4,
      angebotsform = r.angebotsform_3,
      angebotsform_nennung = r.unterstuetzung_angebotsform_sonstiges_nennung,
      -- art_der_unterstuetzung
      -- art_der_unterstuetzung_1
      -- unterstuetzung_sonstiges_nennung_1
      -- angebote_im_bereich
      -- angebot_sonstiges_nennung
      art_der_unterstuetzung_2 = r.art_der_unterstuetzung_2,
      unterstuetzung_sonstiges_nennung_2 = r.unterstuetzung_sonstiges_nennung_2,
      durchfuehrende_institution = r.durchfuehrende_institution_3,

      zielsetzungangebot = r.zielsetzungangebot,
      indikator_1 = r.indikator_1,
      bewertung_manager_1 = r.bewertung_manager_1,
      bewertung_jugendlicher_1 = r.bewertung_jugendlicher_1,
      indikator_2 = r.indikator_2,
      bewertung_manager_2 = r.bewertung_manager_2,
      bewertung_jugendlicher_2 = r.bewertung_jugendlicher_2,
      indikator_3 = r.indikator_3,
      bewertung_manager_3 = r.bewertung_manager_3,
      bewertung_jugendlicher_3 = r.bewertung_jugendlicher_3,
      zielerreichung = r.zielerreichung,

      unterstuetzung_start = r.unterstuetzung_start_3,
      unterstuetzung_ende = r.unterstuetzung_ende_3,
      zeitlicher_umfang_unterstuetzung = r.zeitlicher_umfang_unterstuetzung_3,
      beendigung_1 = r.beendigung_4,
      beendigung_durch = r.beendigung_durch_3,
      gruende_vorzeitige_beendigung = r.gruende_vorzeitige_beendigung_3,
      gruende_sonstiges_nennung = r.gruende_sonstiges_nennung_3,
      nachweis_bescheinigung = r.nachweis_bescheinigung_3,
      sonstiges_nachweis_bescheinigung = r.sonstiges_nachweis_bescheinigung_3
      WHERE id = rgid;
  END LOOP;
END;
$$ LANGUAGE plpgsql;
SET ROLE postgres;
DELETE FROM rg_unterstuetzungsangebote_tbl;
SELECT insert_rg();
SET ROLE :db_adm;
DROP FUNCTION insert_rg();

-- Extra Behandlung der angebotsform
UPDATE rg_unterstuetzungsangebote_tbl set angebotsform = 1 where art_angebot = 4 and angebotsform = 0; 
UPDATE rg_unterstuetzungsangebote_tbl set angebotsform = 4 where art_angebot = 4 and angebotsform = 2;
UPDATE rg_unterstuetzungsangebote_tbl set angebotsform = 5 where art_angebot = 4 and angebotsform = 3;

-- Extra Behandlung der Nachweise (sind nicht einheitlich)
-- Alt BB
-- <option value="0">Schulzeugnis</option>
-- <option value="1">Zertifikat (bewertete Leistung)</option>
-- <option value="2">Teilnahmebescheinigung</option>
-- <option value="3">keine Bescheinigung</option>
-- <option value="-2">Sonstiges</option>
-- Neu BB 
-- <option value="0">Schulzeugnis</option>
-- <option value="1" selected="selected">Zertifikat (bewertete Leistung)</option>
-- <option value="2">Teilnahmebescheinigung</option>
-- <option value="3">keine Bescheinigung</option>


-- Alt BV 
-- <option value="0">Schulzeugnis</option>
-- <option value="1">Zertifikat (bewertete Leistung)</option>
-- <option value="2">Teilnahmebescheinigung</option>
-- <option value="3">keine Bescheinigung</option>
-- Neu BV 
-- <option value="0">Schulzeugnis</option>
-- <option value="1" selected="selected">Zertifikat (bewertete Leistung)</option>
-- <option value="2">Teilnahmebescheinigung</option>
-- <option value="3">keine Bescheinigung</option>

-- Alt BQ
-- <option value="0">zertifizierter Qualifizierungsbaustein</option>
-- <option value="1">Zertifikat/Leistungsnachweis</option>
-- <option value="2">Teilnahmebescheinigung</option>
-- <option value="3">keine Bescheinigung</option>
-- Neu BQ 
-- <option value="0">Schulzeugnis</option>
-- <option value="1" selected="selected">Zertifikat (bewertete Leistung)</option>
-- <option value="2">Teilnahmebescheinigung</option>
-- <option value="3">keine Bescheinigung</option>
UPDATE rg_unterstuetzungsangebote_tbl 
set nachweis_bescheinigung = -2, sonstiges_nachweis_bescheinigung = 'zertifizierter Qualifizierungsbaustein'
where art_angebot = 3 and nachweis_bescheinigung = 0;


-- Alt LB
-- <option value="0">Teilnahmebescheinigung</option>
-- <option value="1">keine Bescheinigung</option>
-- <option value="2">Bestätigung</option>
-- <option value="-2">Sonstiges</option>
-- Neu LB
-- <option value="0">Schulzeugnis</option>
-- <option value="1" selected="selected">Zertifikat (bewertete Leistung)</option>
-- <option value="2">Teilnahmebescheinigung</option>
-- <option value="3">keine Bescheinigung</option>
UPDATE rg_unterstuetzungsangebote_tbl set nachweis_bescheinigung = 2 where art_angebot = 4 and nachweis_bescheinigung = 0;
UPDATE rg_unterstuetzungsangebote_tbl set nachweis_bescheinigung = 3 where art_angebot = 4 and nachweis_bescheinigung = 1;



-- Restore unmodified create_rg_* functions without DSID checks.
CREATE OR REPLACE FUNCTION create_rg_unterstuetzungsangebote_tbl(pid integer, nuuid char(36))
RETURNS integer
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$
DECLARE
    rid integer;
    mid integer;
BEGIN
    mid := pid;

    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    INSERT INTO rg_unterstuetzungsangebote_tbl (master_id) VALUES (pid);

    SELECT INTO rid currval('rg_unterstuetzungsangebote_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE rg_unterstuetzungsangebote_tbl
        SET    uuid_id = nuuid
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('rg_unterstuetzungsangebote_tbl', rid);
        
    END IF;

    RETURN rid;
END;
$$;


SET ROLE postgres;
ALTER TABLE master_tbl ENABLE TRIGGER USER;

COMMIT;
