BEGIN;
-- waska_create_role( nutzer, gruppenrolle, agentur ) returns text 
-- Anlegen einer Login-Rolle 'ka_<agentur>_<nutzer>' in der gruppenrolle:
-- 'CM-KA' oder 'Admin-KA'. Es wird anhand der Namensräume überprüft, ob die
-- aufrufende Rolle berechtigt ist, für die angegebene KA Nutzer anzulegen.
-- Rückgabewert ist die Loginrolle und das initiale Passwort.
CREATE OR REPLACE FUNCTION waska_create_role (
        username varchar(64),
        grouprole varchar(64), 
        agency varchar(52)
  ) RETURNS RECORD LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
  AS $$
  DECLARE 
    db_name  varchar(64);
    db_user  varchar(128);
    db_group varchar(64);
    pw       varchar(9);
    result   RECORD;
  BEGIN
    -- IF position(quote_literal(agency) in session_user) != 4
    IF position(agency in session_user) != 4
    THEN
      RAISE EXCEPTION 
          'Permission denied to create role %s in agency % by %.',
          quote_literal(username), quote_literal(agency), session_user;
    ELSE
      db_name := quote_ident('ka_' || agency || '_db');
      db_user := quote_ident('ka_' || agency || '_' || username);
      IF lower(grouprole) = 'admin-ka' THEN
        db_group := quote_ident('ka_' || agency || '_admin_ka');
      ELSIF lower(grouprole) = 'pb-ka' THEN
        db_group := quote_ident('ka_' || agency || '_pb_ka');
      ELSE
        db_group := quote_ident('ka_' || agency || '_cm_ka');
      END IF;
      pw := waska_passgen(8);
      EXECUTE 'CREATE ROLE ' || db_user 
          || ' LOGIN NOSUPERUSER INHERIT NOCREATEDB NOCREATEROLE'
          || ' PASSWORD ' || quote_literal(pw);
      EXECUTE 'GRANT ' || db_group || ' TO ' || db_user;
      SELECT INTO result db_name::varchar(64), db_user::varchar(64), pw::varchar(9);
      RETURN result;
    END IF;

  END;
$$ ;
REVOKE ALL ON FUNCTION waska_create_role( varchar, varchar, varchar ) 
    FROM public;
GRANT EXECUTE ON FUNCTION waska_create_role( varchar, varchar, varchar ) 
    TO admin_ka;

UPDATE pg_database SET datallowconn = FALSE where datname='waska_template';
COMMIT;

-- Erstellen der neuen PB - Rolle
BEGIN;

CREATE ROLE :pb_ka_group NOSUPERUSER INHERIT NOCREATEDB NOCREATEROLE;
GRANT :pb_ka_group TO :admin_ka_group;

COMMIT;
