BEGIN;
SET ROLE :db_adm;

--
-- Function to anonymize a case and all of its repeat groups
--
CREATE OR REPLACE FUNCTION anonymize_case(mid int)
RETURNS void AS
$$
BEGIN
    -- anonymize master table
    UPDATE master_tbl_view SET
        fkz = NULL,
        name_kompetenzagentur = NULL,
        vorname = NULL,
        name = hash_value(name, vorname),
        kundennummer = NULL,
        kontaktdaten_arge = NULL,
        kundennummer_arge = NULL,
        kontaktdaten_agentur_fuer_arbeit = NULL,
        kundennummer_agentur_fuer_arbeit = NULL,
        kundennummer_jugendamt = NULL,
        kontaktdaten_jugendamt = NULL,
        kontaktdaten_schule = NULL,
        schulname = NULL,
        kontaktdaten_gerichtshilfe = NULL,
        kundennummer_gerichtshilfe = NULL,
        kontaktdaten_weitere_institutionen = NULL,
        vorname_1 = NULL,
        name_1 = NULL,
        geburtsdatum = get_age(geburtsdatum),
        strasse = NULL,
        plz = shorten_value(plz, 3),
        wohnort = NULL,
        vorwahl = NULL,
        telefon_1 = NULL,
        mobiltelefon = NULL,
        email = NULL,
        telefon_sonstige = NULL,
        zweiter_wohnort = NULL,
        geburtsland_sonstige_nennung = NULL,
        muttersprache_sonstige_nennung = NULL,
        staatsangehoerigkeit_sonstige_nennung = NULL,
        vorname_2 = NULL,
        name_2 = NULL,
        vorwahl_1 = NULL,
        telefon_1_1 = NULL,
        mobiltelefon_1 = NULL,
        email_1 = NULL,
        physische_beeintraechtigungen = check_bool(physische_beeintraechtigungen, arzt_gutachten_physisch),
        physische_beeintraechtigungen_text = check_bool(physische_beeintraechtigungen_text, arzt_gutachten_physisch),
        psychische_beeintraechtigungen = check_bool(psychische_beeintraechtigungen, arzt_gutachten_psychisch),
        psychische_beeintraechtigungen_text = check_bool(psychische_beeintraechtigungen_text, arzt_gutachten_psychisch),
        schulabschlusserwerbsland_sonstige_nennung = NULL,
        abgebrochene_ausbildungen_text = NULL,
        weitere_ausbildungen_text_1 = NULL,
        abgebrochene_ausbildungen_text_1 = NULL,
        letzte_massnahme = NULL,
        letzte_taetigkeit = NULL,
        freie_dokumentation = NULL,
        sonstiger_foerderbedarf_nennung = NULL,
        dokumentation_zum_foerderplan = NULL,
        gruende_sonstiges_nennung_4 = NULL,
        klassenstufe_1 = NULL,
        freie_dokumentation_abschlussbewertung = NULL
    WHERE id = mid;

    -- anonymize repeat groups
    UPDATE rg_kompetenzfestellung_tbl_view SET
        dokumentation_kompetenzfeststellung = NULL
    WHERE master_id = mid;

    UPDATE rg_angebote_bildenden_bereich_tbl_view SET
        gruende_sonstiges_nennung = NULL,
        sonstiges_nachweis_bescheinigung = NULL
    WHERE master_id = mid;

    UPDATE rg_angebote_berufsvorbereitung_tbl_view SET
        gruende_sonstiges_nennung_1 = NULL,
        sonstiges_nachweis_bescheinigung_1 = NULL
    WHERE master_id = mid;

    UPDATE rg_angebote_berufliche_qualifizierung_tbl_view SET
        gruende_sonstiges_nennung_2 = NULL,
        sonstiges_nachweis_bescheinigung_2 = NULL
    WHERE master_id = mid;

    UPDATE rg_angebote_lebensbewaeltigung_tbl_view SET
        gruende_sonstiges_nennung_3 = NULL,
        sonstiges_nachweis_bescheinigung_3 = NULL
    WHERE master_id = mid;

END;
$$ LANGUAGE plpgsql;

REVOKE ALL ON FUNCTION anonymize_case(int) FROM public;
GRANT EXECUTE ON FUNCTION anonymize_case(int) TO GROUP :cm_ka_group;

COMMIT;

