# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS -
# Web-Anwendungs-Server fuer Kompetenzagenturen).
#
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls WASKA has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.
#
# Authors:
# Torsten Irländer <torsten.irlaender@intevation.de>
# Sascha L. Teichmann <teichmann@intevation.de>
#

import logging

import formencode

from mpulsweb.lib.base import c, g, render, request, session
from mpulsweb.lib.security import checkRole

from mpulsweb.model.case import SessionCase, Agency
from mpulsweb.model.statement import PrivacyStatement

from mpulsweb.controllers.formularpage import convert2dic
from mpulsweb.controllers.navigation import NavigationController 
from mpulsweb.controllers.case import CaseController as MpulsCaseController

from waskaweb.lib.validators import NewCaseForm

log = logging.getLogger(__name__)

class CaseController(MpulsCaseController):

    @checkRole('cm_ka')
    def newAction(self):
        # TODO: Use htmlfill here!
        session_case = SessionCase()
        session_case.setData(request.params)

        session['case'] = session_case
        session.save()
        agency =  Agency()

        # Do we want to print the statement?
        if request.params.has_key('print'):
            c.print_version = 1
            statement = PrivacyStatement(session_case, agency)
            c.content = statement.fillout()
            return render('privacy/default_statement.mako')

        # Or do we want to store the new case?
        elif request.params.has_key('save'):
            validator = NewCaseForm()
            try:
                form_result = validator.to_python(request.params)
                # Add fkz of the agency to the form_result
                form_result['fkz'] = agency.getFKZ() 
                # 1. Create a new case
                case = g.case_factory.createNew(data=convert2dic({}))
                case._load_formed_instance_tree()
                # Set data
                data = {}
                for k in convert2dic(form_result).keys():
                    try:
                        item = case.formed_instance.getAllItemsForName(k)[0]
                        id = ":".join(item.getIdentifier().split(':')[1:])
                        data["%s:%s" % (k,id)] = form_result.get(k) 
                    except:
                        # some values within the formresult can not be found in
                        # instance tree. e.g "statement_signed"
                        pass
                case.formed_instance.setData(data)
                case.formed_instance.commit()

                statement = case.getPrivacyStatement()
                # 3. Do we want to sign the statement?
                if form_result.get('statement_signed'):
                    statement.sign()

            except formencode.Invalid, error:
                log.debug("Invalid form data: %r", error)
                c.form_result = error.value
                c.form_errors = error.error_dict or {}
                return render('/casemanagement/new.mako')

        return self.select(str(case.id))

    def print_AidplanStatement(self, id):
        # Call navigation controller which will trigger rendering the right
        # formularpage. Content of the formularpage (c.formular) will be user
        # to render the aidplan.
        NavigationController().select_branch('rg_unterstuetzungsangebote:%s' % id)
        c.case = self.init_case(id)
        return render('/casemanagement/aidplan.mako')

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
