# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS -
# Web-Anwendungs-Server fuer Kompetenzagenturen).
#
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls WASKA has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.
#
# Authors:
# Torsten Irländer <torsten.irlaender@intevation.de>

import logging

from formencode import ForEach
from formencode.validators import Int

from mpulsweb.lib.search import CaseSearch as CaseBaseSearch


log = logging.getLogger(__name__)


class CaseSearch(CaseBaseSearch):

    VALID_OPTIONS = (CaseBaseSearch.VALID_OPTIONS
                     + ("gender", "type_ending"))
    MULTI_OPTIONS = (CaseBaseSearch.MULTI_OPTIONS
                     + ("type_ending",))

    def validate_form_params(self, params):
        # FIXME: We import SearchCaseForm here because importing it
        # during import doesn't work.  The validators module performs
        # translations and accesses app_globals during import, so it has
        # to be imported when a HTTP-request is being handled.  The
        # search module is imported early on when the application starts
        # so validators cannot be imported then.
        from mpulsweb.lib.validators import SearchCaseForm
        validator = SearchCaseForm(type_ending=ForEach(Int(),
                                                       convert_to_list=True))
        return validator.to_python(params)

    def convert_form_parameters(self, options, user, extended):
        search = CaseBaseSearch.convert_form_parameters(self, options, user,
                                                        extended=extended)
        if extended:
            type_ending = options.get("type_ending")
            if type_ending:
                search["type_ending"] = list(type_ending)

        return search

    def get_ending_clause(self, options):
        """Return the where subclause that selects based on CM ending.
        If the type_ending option was not specified, any ending will be
        OK, so this method returns 'TRUE'.  Otherwise only endings
        contained in the list value of the type_ending option will be
        selected.  If that list is empty, no ending will be OK, so no
        case will be found.
        """
        return self.generic_int_choice_clause(options, "type_ending",
                                              "m.art_beendigung")

    def get_gender_clause(self, options):
        gender = options.get("gender")
        if gender is not None:
            return "m.geschlecht = %s" % gender
        return "TRUE"

    def get_where_subclauses(self, options):
        return CaseBaseSearch.get_where_subclauses(self, options) + [
            self.get_ending_clause(options),
            ]

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
